@@ -1,4 +1,4 @@
-# Copyright (C) 2003, 2004 by Intevation GmbH
+# Copyright (C) 2003, 2004, 2005 by Intevation GmbH
 # Authors:
 # Martin Mueller <mmueller@intevation.de>
 # Bernhard Herzog <bh@intevation.de>
@@ -32,10 +32,23 @@ def psycopg_version():
     return psycopg.__version__
 
 if psycopg is not None:
+    # type_map maps psycopg type objects.  It's a list of pairs since
+    # the psycopg type objects are unhashable.
     type_map = [(psycopg.STRING, table.FIELDTYPE_STRING),
                 (psycopg.INTEGER, table.FIELDTYPE_INT),
+                (psycopg.ROWID, table.FIELDTYPE_INT),
                 (psycopg.FLOAT, table.FIELDTYPE_DOUBLE)]
 
+    # _raw_type_map maps the postgresql type constants to Thuban type
+    # constants.  This is very low level and postgresql specific and
+    # should be used only when necessary.
+    _raw_type_map = {}
+    def _fill_raw_type_map():
+        for psycopg_type, thuban_type in type_map:
+            for value in psycopg_type.values:
+                _raw_type_map[value] = thuban_type
+    _fill_raw_type_map()
+
 
 def quote_identifier(ident):
     """Return a quoted version of the identifier ident.
@@ -94,6 +107,16 @@ class PostGISConnection:
         except psycopg.OperationalError, val:
             raise ConnectionError(str(val))
 
+        # Use autocommit mode.  For simple reading of the database it's
+        # sufficient and we don't have to care much about error
+        # handling.  Without autocommit, an errors during a cursor's
+        # execute method requires a rollback on the connection,
+        # otherwise later queries with the same or other cursors sharing
+        # the same connection will lead to further errors ("ERROR:
+        # current transaction is aborted, commands ignored until end of
+        # transaction block")
+        self.connection.autocommit()
+
         # determine the OID for the geometry type. This is PostGIS
         # specific.
         cursor = self.connection.cursor()
@@ -135,12 +158,67 @@ class PostGISConnection:
 
     def GeometryTables(self):
         """Return a list with the names of all tables with a geometry column"""
+
+        # The query is basically taken from the psql v. 7.2.1.  When
+        # started with -E it prints the queries used for internal
+        # commands such as \d, which does mostly what we need here.
         cursor = self.connection.cursor()
-        cursor.execute("SELECT f_table_name FROM geometry_columns;")
+        cursor.execute("SELECT c.relname FROM pg_class c"
+                       " WHERE c.relkind IN ('r', 'v')"
+                             # Omit the system tables
+                             " AND c.relname !~ '^pg_'"
+                             # Omit the special PostGIS tables
+                             " AND c.relname NOT IN ('geometry_columns',"
+                                                   " 'spatial_ref_sys')"
+                            " AND %d in (SELECT a.atttypid FROM pg_attribute a"
+                                       " WHERE a.attrelid = c.oid)"
+                       " ORDER BY c.relname;", (self.geometry_type,))
         result = [row[0] for row in cursor.fetchall()]
         self.connection.commit()
         return result
 
+    def table_columns(self, tablename):
+        """Experimental: return information about the columns of a table
+
+        Return value is a list of (name, type) pairs where name is the
+        name of the column and type either one of the field type columns
+        or the string 'geometry' indicating a geometry column.
+
+        The intended use of this method is for table selection dialogs
+        which need to determine which columns are usable as id or
+        geometry columns respectively.  Suitable id columns will have
+        type FIELDTYPE_INT and geometry columns will have 'geometry'.
+        """
+        result = []
+        cursor = self.connection.cursor()
+
+        # This query is taken basically from the \d command of psql
+        # 7.2.1
+        cursor.execute("SELECT a.attname, a.atttypid, a.attnum"
+                       " FROM pg_class c, pg_attribute a"
+                            " WHERE c.relname = %s AND a.attrelid = c.oid"
+                       " ORDER BY a.attnum;", (tablename,))
+
+        for row in cursor.fetchall():
+            col_name, col_type, col_attnum = row
+            col = None
+            if col_attnum < 1:
+                # It's a system column.  Only the OID is interesting
+                # here
+                if col_name == "oid":
+                    col = (col_name, _raw_type_map[col_type])
+            else:
+                # If it's an integer
+                thuban_type = _raw_type_map.get(col_type)
+                if thuban_type is not None:
+                    col = (col_name, thuban_type)
+                elif row[1] == self.geometry_type:
+                    col = (col_name, "geometry")
+            if col is not None:
+                result.append(col)
+
+        return result
+
     def cursor(self):
         """Return a DB API 2.0 cursor for the database"""
         return self.connection.cursor()
@@ -175,17 +253,32 @@ class PostGISTable:
     descriptions returned by Columns() and other methods.
     """
 
-    def __init__(self, db, tablename):
+    def __init__(self, db, tablename, id_column = None):
         """Initialize the PostGISTable.
 
         The db parameter should be an instance of PostGISConnection and
         tablename the name of a table in the database represented by db.
+
+        The id_column parameter should be the name of a column in the
+        table that can be used to identify rows.  The column must have
+        the type integer and be unique and not null.
+
+        For backwards compatibility reasons, the id_column parameter is
+        optional.  If not given the table must have a column called
+        'gid' which is used as the id_column.  New code should always
+        provide this parameter.
         """
         self.db = db
         self.tablename = tablename
         # Tablename quoted for use in SQL statements.
         self.quoted_tablename = quote_identifier(tablename)
 
+        if not id_column:
+            id_column = "gid"
+        self.id_column = id_column
+        # id column name quoted for use in SQL statements.
+        self.quoted_id_column = quote_identifier(id_column)
+
         # Map column names and indices to column objects.
         self.column_map = {}
 
@@ -236,6 +329,18 @@ class PostGISTable:
         """Return the dbconnection used by the table"""
         return self.db
 
+    def IDColumn(self):
+        """Return the column description object for the id column.
+
+        If the oid column was used as the id column, the return value is
+        not one of the regular column objects that would be returned by
+        e.g. the Column() method, but it still has meaningful name
+        attribute.
+        """
+        if self.id_column == "oid":
+            return PostGISColumn(self.id_column, table.FIELDTYPE_INT, None)
+        return self.column_map[self.id_column]
+
     def TableName(self):
         """Return the name of the table in the database"""
         return self.tablename
@@ -272,15 +377,15 @@ class PostGISTable:
     def RowIdToOrdinal(self, gid):
         """Return the row ordinal given its id"""
         cursor = self.db.cursor()
-        cursor.execute("SELECT count(*) FROM %s WHERE gid < %d;"
-                       % (self.quoted_tablename, gid))
+        cursor.execute("SELECT count(*) FROM %s WHERE %s < %d;"
+                       % (self.quoted_tablename, self.quoted_id_column, gid))
         return cursor.fetchone()[0]
 
     def RowOrdinalToId(self, num):
         """Return the rowid for given its ordinal"""
         cursor = self.db.cursor()
-        cursor.execute("SELECT gid FROM %s LIMIT 1 OFFSET %d;"
-                       % (self.quoted_tablename, num))
+        cursor.execute("SELECT %s FROM %s LIMIT 1 OFFSET %d;"
+                       % (self.quoted_id_column, self.quoted_tablename, num))
         return cursor.fetchone()[0]
 
     def ReadRowAsDict(self, row, row_is_ordinal = 0):
@@ -288,7 +393,8 @@ class PostGISTable:
         if row_is_ordinal:
             stmt = self.query_stmt + " LIMIT 1 OFFSET %d" % row
         else:
-            stmt = self.query_stmt + " WHERE gid = %d" % row
+            stmt = self.query_stmt + " WHERE %s = %d" % (self.quoted_id_column,
+                                                         row)
         cursor.execute(stmt)
         result = {}
         for col, value in zip(self.columns, cursor.fetchone()):
@@ -302,9 +408,9 @@ class PostGISTable:
                     (self.column_map[col].quoted_name, self.quoted_tablename,
                      row))
         else:
-            stmt = ("SELECT %s FROM %s WHERE gid = %d" %
+            stmt = ("SELECT %s FROM %s WHERE %s = %d" %
                     (self.column_map[col].quoted_name, self.quoted_tablename,
-                     row))
+                     self.quoted_id_column, row))
         cursor.execute(stmt)
         return cursor.fetchone()[0]
 
@@ -336,9 +442,10 @@ class PostGISTable:
             right_template = "%s"
             params = (right,)
 
-        query = "SELECT gid FROM %s WHERE %s %s %s ORDER BY gid;" \
-                % (self.quoted_tablename, left.quoted_name, comparison,
-                   right_template)
+        query = "SELECT %s FROM %s WHERE %s %s %s ORDER BY %s;" \
+                % (self.quoted_id_column, self.quoted_tablename,
+                   left.quoted_name, comparison, right_template,
+                   self.quoted_id_column)
 
         cursor = self.db.cursor()
         cursor.execute(query, params)
@@ -390,12 +497,17 @@ class PostGISShapeStore(PostGISTable):
 
     """Shapestore interface to a table in a PostGIS database"""
 
-    def __init__(self, db, tablename, geometry_column = None):
+    def __init__(self, db, tablename, id_column = "gid",
+                 geometry_column = None):
         """Initialize the PostGISShapeStore.
 
         The db parameter should be an instance of PostGISConnection and
         tablename the name of a table in the database represented by db.
 
+        The id_column parameter should be the name of a column in the
+        table that can be used to identify rows.  The column must have
+        the type integer and be unique and not null.
+
         The geometry_column paramter, if given, should be the name of
         the geometry column to use.  If the name given is not a geometry
         column, raise a ValueError.
@@ -406,25 +518,61 @@ class PostGISShapeStore(PostGISTable):
         """
         self.geometry_column = geometry_column
         self.geometry_column_was_given = geometry_column is not None
-        PostGISTable.__init__(self, db, tablename)
+        PostGISTable.__init__(self, db, tablename, id_column)
 
         # For convenience, we have a quoted version of the geometry
         # column in self.quoted_geo_col
         self.quoted_geo_col = quote_identifier(self.geometry_column)
 
     def _fetch_table_information(self):
-        """Extend inherited method to retrieve the SRID"""
+        """Extend inherited method to retrieve the SRID and shape type"""
         PostGISTable._fetch_table_information(self)
+
+        # First, try to get it from the geometry_columns table.
         cursor = self.db.cursor()
-        cursor.execute("SELECT srid FROM geometry_columns"
+        cursor.execute("SELECT srid, type FROM geometry_columns"
                        " WHERE f_table_name = %s AND f_geometry_column=%s",
                        (self.tablename, self.geometry_column))
-        self.srid = cursor.fetchone()[0]
+        row = cursor.fetchone()
+        if row is not None:
+            self.srid = row[0]
+            self.shape_type = shapetype_map.get(row[1])
+            return
+
+        # The table is probably really a view and thus not in
+        # geometry_columns.  Use a different approach
+        cursor = self.db.cursor()
+        cursor.execute("SELECT DISTINCT SRID(%s) FROM %s;" %
+                       (quote_identifier(self.geometry_column),
+                        self.tablename))
+        row = cursor.fetchone()
+        if row is not None:
+            self.srid = row[0]
+            # Try to see whether there's another one
+            row = cursor.fetchone()
+            if row is not None:
+                # There are at least two different srids.  We don't
+                # support that
+                self.srid = None
+
+        cursor = self.db.cursor()
+        cursor.execute("SELECT DISTINCT GeometryType(%s) FROM %s;"
+                       % (quote_identifier(self.geometry_column),
+                          self.tablename))
+        row = cursor.fetchone()
+        if row is not None:
+            self.shape_type = shapetype_map.get(row[0])
+            # Try to see whether there's another one
+            row = cursor.fetchone()
+            if row is not None:
+                # There are at least two different srids.  We don't
+                # support that
+                self.shape_type = None
 
     def _create_col_from_description(self, index, description):
         """Extend the inherited method to find geometry columns
 
-        If the column indicated by the paramters is a geometry column,
+        If the column indicated by the parameters is a geometry column,
         record its name in self.geometry_column and a quoted version in
         self.quoted_geo_col.  In any case return the return value of the
         inherited method.
@@ -461,15 +609,19 @@ class PostGISShapeStore(PostGISTable):
         """
         return None
 
+    def GeometryColumn(self):
+        """Return the column description object for the geometry column
+
+        There's currently no FIELDTYPE constant for this column, so the
+        return value is not a regular column object that could also be
+        returned from e.g. the Column() method.  Only the name attribute
+        of the return value is meaningful at the moment.
+        """
+        return PostGISColumn(self.geometry_column, None, None)
+
     def ShapeType(self):
         """Return the type of the shapes in the shapestore."""
-        cursor = self.db.cursor()
-        cursor.execute("SELECT type FROM geometry_columns WHERE"
-                       " f_table_name=%s AND f_geometry_column=%s",
-                       (self.tablename, self.geometry_column))
-        result = cursor.fetchone()[0]
-        cursor.close()
-        return shapetype_map[result]
+        return self.shape_type
 
     def RawShapeFormat(self):
         """Return the raw data format of the shape data.
@@ -506,16 +658,18 @@ class PostGISShapeStore(PostGISTable):
 
     def Shape(self, shapeid):
         cursor = self.db.cursor()
-        cursor.execute("SELECT AsText(%s) FROM %s WHERE gid=%d"
-                       % (self.quoted_geo_col, self.quoted_tablename, shapeid))
+        cursor.execute("SELECT AsText(%s) FROM %s WHERE %s=%d"
+                       % (self.quoted_geo_col, self.quoted_tablename,
+                          self.quoted_id_column, shapeid))
         wkt = cursor.fetchone()[0]
         cursor.close()
         return PostGISShape(shapeid, wkt)
 
     def AllShapes(self):
         cursor = self.db.cursor()
-        cursor.execute("SELECT gid, AsText(%s) FROM %s ORDER BY gid"
-                       % (self.quoted_geo_col, self.quoted_tablename))
+        cursor.execute("SELECT %s, AsText(%s) FROM %s ORDER BY %s"
+                       % (self.quoted_id_column, self.quoted_geo_col,
+                          self.quoted_tablename, self.quoted_id_column))
         while 1:
             result = cursor.fetchone()
             if result is None:
@@ -531,10 +685,14 @@ class PostGISShapeStore(PostGISTable):
                 % (left, bottom, left, top, right, top, right, bottom,
                    left, bottom))
         cursor = self.db.cursor()
-        cursor.execute("SELECT gid, AsText(%s) FROM %s"
-                     " WHERE %s && GeometryFromText('%s', %d) ORDER BY gid"
-                       % (self.quoted_geo_col, self.quoted_tablename,
-                          self.quoted_geo_col, geom, self.srid))
+        cursor.execute("SELECT %(gid)s, AsText(%(geom)s) FROM %(table)s"
+                     " WHERE %(geom)s && GeometryFromText('%(box)s', %(srid)d)"
+                       " ORDER BY %(gid)s"
+                       % {"table": self.quoted_tablename,
+                          "geom": self.quoted_geo_col,
+                          "gid": self.quoted_id_column,
+                          "box": geom,
+                          "srid": self.srid})
         while 1:
             result = cursor.fetchone()
             if result is None:
