@@ -7,14 +7,14 @@
 
 """The Classification Generator Dialog"""
 
-__version__ = "$Revision: 1.30.2.4 $"
-# $Source: /thubanrepository/thuban/Thuban/UI/classgen.py,v $
-# $Id: classgen.py,v 1.30.2.4 2004/12/15 16:45:30 bh Exp $
+__version__ = "$Revision: 2709 $"
+# $Source$
+# $Id: classgen.py 2709 2006-10-03 09:34:51Z dpinte $
 
 
 import sys
 
-from wxPython.wx import *
+import wx
 
 from Thuban import _
 
@@ -58,7 +58,7 @@ ID_CLASSGEN_PROPCOMBO = 4008
 ID_BORDER_COLOR = 4009
 ID_BORDER_COLOR_CHANGE = 4010
 
-class ClassGenDialog(wxDialog):
+class ClassGenDialog(wx.Dialog):
 
     def __init__(self, parent, layer, fieldName):
         """Inialize the class generating dialog.
@@ -66,8 +66,8 @@ class ClassGenDialog(wxDialog):
         parent -- this must be an instance of the Classifier class
         """
 
-        wxDialog.__init__(self, parent, -1, _("Generate Classification"),
-                          style = wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER)
+        wx.Dialog.__init__(self, parent, -1, _("Generate Classification"),
+                          style = wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER)
 
         self.parent = parent
         self.layer = layer
@@ -87,11 +87,11 @@ class ClassGenDialog(wxDialog):
         # we need to create genButton first because when we create the
         # panels they will call AllowGenerate() which uses genButton.
         #
-        self.genButton = wxButton(self, wxID_OK, _("Generate")) 
-        self.cancelButton = wxButton(self, wxID_CANCEL, _("Close")) 
+        self.genButton = wx.Button(self, wx.ID_OK, _("Generate"))
+        self.cancelButton = wx.Button(self, wx.ID_CANCEL, _("Close"))
         self.genButton.SetDefault()
 
-        self.genChoice = wxChoice(self, ID_CLASSGEN_GENCOMBO)
+        self.genChoice = wx.Choice(self, ID_CLASSGEN_GENCOMBO)
 
         self.genpanels.append((GENCOMBOSTR_UNIQUE, GenUniquePanel))
         if self.type in (FIELDTYPE_INT, FIELDTYPE_DOUBLE):
@@ -114,31 +114,31 @@ class ClassGenDialog(wxDialog):
 
         #############
 
-        sizer = wxBoxSizer(wxVERTICAL)
+        sizer = wx.BoxSizer(wx.VERTICAL)
 
-        sizer.Add(wxStaticText(self, -1, _("Field: %s") % fieldName), 
-                  0, wxALL, 4)
-        sizer.Add(wxStaticText(
-            self, -1, 
+        sizer.Add(wx.StaticText(self, -1, _("Field: %s") % fieldName),
+                  0, wx.ALL, 4)
+        sizer.Add(wx.StaticText(
+            self, -1,
             _("Data Type: %s") % classifier.Classifier.type2string[self.type]),
-            0, wxALL, 4)
+            0, wx.ALL, 4)
 
-        psizer = wxBoxSizer(wxHORIZONTAL)
-        psizer.Add(wxStaticText(self, -1, _("Generate:")),
-            0, wxALIGN_CENTER_VERTICAL, 0)
-        psizer.Add(self.genChoice, 1, wxALL | wxGROW, 4)
+        psizer = wx.BoxSizer(wx.HORIZONTAL)
+        psizer.Add(wx.StaticText(self, -1, _("Generate:")),
+            0, wx.ALIGN_CENTER_VERTICAL, 0)
+        psizer.Add(self.genChoice, 1, wx.ALL | wx.GROW, 4)
 
-        sizer.Add(psizer, 0, wxALL | wxGROW, 4)
+        sizer.Add(psizer, 0, wx.ALL | wx.GROW, 4)
 
-        self.sizer_genPanel = wxBoxSizer(wxVERTICAL)
-        sizer.Add(self.sizer_genPanel, 1, wxGROW | wxALL, 4)
+        self.sizer_genPanel = wx.BoxSizer(wx.VERTICAL)
+        sizer.Add(self.sizer_genPanel, 1, wx.GROW | wx.ALL, 4)
 
-        psizer = wxBoxSizer(wxHORIZONTAL)
-        psizer.Add(wxStaticText(self, -1, _("Color Scheme:")),
-            0, wxALIGN_CENTER_VERTICAL, 0)
+        psizer = wx.BoxSizer(wx.HORIZONTAL)
+        psizer.Add(wx.StaticText(self, -1, _("Color Scheme:")),
+            0, wx.ALIGN_CENTER_VERTICAL, 0)
 
         # Properties (Ramp) ComboBox
-        self.propCombo = wxChoice(self, ID_CLASSGEN_PROPCOMBO)
+        self.propCombo = wx.Choice(self, ID_CLASSGEN_PROPCOMBO)
 
         self.propPanel = None
         custom_ramp_panel = CustomRampPanel(self, layer.ShapeType())
@@ -153,33 +153,33 @@ class ClassGenDialog(wxDialog):
 
         self.propCombo.SetSelection(0)
 
-        psizer.Add(self.propCombo, 1, wxALL | wxGROW, 4)
-        sizer.Add(psizer, 0, wxALL | wxGROW, 4)
+        psizer.Add(self.propCombo, 1, wx.ALL | wx.GROW, 4)
+        sizer.Add(psizer, 0, wx.ALL | wx.GROW, 4)
 
         if layer.ShapeType() != SHAPETYPE_ARC:
-            psizer = wxBoxSizer(wxHORIZONTAL)
-            self.fix_border_check = wxCheckBox(self, -1, _("Fix Border Color"))
-            psizer.Add(self.fix_border_check, 0, wxALL | wxGROW, 4)
+            psizer = wx.BoxSizer(wx.HORIZONTAL)
+            self.fix_border_check = wx.CheckBox(self, -1, _("Fix Border Color"))
+            psizer.Add(self.fix_border_check, 0, wx.ALL | wx.GROW, 4)
             self.border_color = classifier.ClassGroupPropertiesCtrl(
                 self, ID_BORDER_COLOR,
                 ClassGroupProperties(), SHAPETYPE_ARC,
-                style=wxSIMPLE_BORDER, size=(40, 20))
-            psizer.Add(self.border_color, 0, wxALL | wxGROW, 4)
-            psizer.Add(wxButton(self, ID_BORDER_COLOR_CHANGE, _("Change")),
-                    0, wxALL, 4)
-            sizer.Add(psizer, 0, wxALL | wxGROW, 4)
-            EVT_BUTTON(self, ID_BORDER_COLOR_CHANGE, self.OnBorderColorChange)
+                style=wx.SIMPLE_BORDER, size=(40, 20))
+            psizer.Add(self.border_color, 0, wx.ALL | wx.GROW, 4)
+            psizer.Add(wx.Button(self, ID_BORDER_COLOR_CHANGE, _("Change")),
+                    0, wx.ALL, 4)
+            sizer.Add(psizer, 0, wx.ALL | wx.GROW, 4)
+            self.Bind(wx.EVT_BUTTON, self.OnBorderColorChange, id=ID_BORDER_COLOR_CHANGE)
         else:
             self.border_color = None
 
-        sizer.Add(custom_ramp_panel, 1, wxGROW | wxALL, 4)
+        sizer.Add(custom_ramp_panel, 1, wx.GROW | wx.ALL, 4)
         sizer.Show(custom_ramp_panel, False)
 
         # Finally place the main buttons
-        buttonSizer = wxBoxSizer(wxHORIZONTAL)
-        buttonSizer.Add(self.genButton, 0, wxRIGHT|wxEXPAND, 10)
-        buttonSizer.Add(self.cancelButton, 0, wxRIGHT|wxEXPAND, 10)
-        sizer.Add(buttonSizer, 0, wxALIGN_RIGHT|wxBOTTOM|wxTOP, 10)
+        buttonSizer = wx.BoxSizer(wx.HORIZONTAL)
+        buttonSizer.Add(self.genButton, 0, wx.RIGHT|wx.EXPAND, 10)
+        buttonSizer.Add(self.cancelButton, 0, wx.RIGHT|wx.EXPAND, 10)
+        sizer.Add(buttonSizer, 0, wx.ALIGN_RIGHT|wx.BOTTOM|wx.TOP, 10)
 
         self.SetSizer(sizer)
         self.SetAutoLayout(True)
@@ -189,10 +189,10 @@ class ClassGenDialog(wxDialog):
 
         self.__DoOnGenTypeSelect()
 
-        EVT_CHOICE(self, ID_CLASSGEN_GENCOMBO, self._OnGenTypeSelect)
-        EVT_CHOICE(self, ID_CLASSGEN_PROPCOMBO, self._OnPropTypeSelect)
-        EVT_BUTTON(self, wxID_OK, self.OnOK)
-        EVT_BUTTON(self, wxID_CANCEL, self.OnCancel)
+        self.Bind(wx.EVT_CHOICE, self._OnGenTypeSelect, id=ID_CLASSGEN_GENCOMBO)
+        self.Bind(wx.EVT_CHOICE, self._OnPropTypeSelect, id=ID_CLASSGEN_PROPCOMBO)
+        self.Bind(wx.EVT_BUTTON, self.OnOK, self.genButton)
+        self.Bind(wx.EVT_BUTTON, self.OnCancel, self.cancelButton)
 
         self.__DoOnGenTypeSelect()
 
@@ -232,7 +232,7 @@ class ClassGenDialog(wxDialog):
             ramp = propPanel.GetRamp()
             if self.border_color and self.fix_border_check.IsChecked():
                 props = self.border_color.GetProperties()
-                ramp = FixedRamp(ramp, 
+                ramp = FixedRamp(ramp,
                     (props.GetLineColor(), props.GetLineWidth(), None))
 
             if genSel == GENCOMBOSTR_UNIFORM:
@@ -270,14 +270,14 @@ class ClassGenDialog(wxDialog):
                     generate_quantiles(_list, percents, ramp, _range)
 
                 if adjusted:
-                    dlg = wxMessageDialog(self, 
+                    dlg = wx.MessageDialog(self,
                         _("Based on the data from the table and the input\n"
                      "values, the exact quantiles could not be generated.\n\n"
                           "Accept a close estimate?"),
                         _("Problem with Quantiles"),
 
-                        wxYES_NO|wxYES_DEFAULT|wxICON_QUESTION)
-                    if dlg.ShowModal() == wxID_YES:
+                        wx.YES_NO|wx.YES_DEFAULT|wx.ICON_QUESTION)
+                    if dlg.ShowModal() == wx.ID_YES:
                         self.parent._SetClassification(self.clazz)
                 else:
                     self.parent._SetClassification(self.clazz)
@@ -312,12 +312,12 @@ class ClassGenDialog(wxDialog):
         selIndex = combo.GetSelection()
         sel = combo.GetString(selIndex)
 
-        if isinstance(self.propPanel, wxPanel):
+        if isinstance(self.propPanel, wx.Panel):
             self.topBox.Show(self.propPanel, False)
 
         self.propPanel = combo.GetClientData(selIndex)
 
-        if isinstance(self.propPanel, wxPanel):
+        if isinstance(self.propPanel, wx.Panel):
             self.topBox.Show(self.propPanel, True)
 
         self.topBox.SetSizeHints(self)
@@ -339,7 +339,7 @@ class ClassGenDialog(wxDialog):
         self.curGenPanel.Show()
 
         self.sizer_genPanel.Add(self.curGenPanel, 1,
-            wxALL|wxEXPAND|wxADJUST_MINSIZE, 3)
+            wx.ALL|wx.EXPAND|wx.ADJUST_MINSIZE, 3)
         self.sizer_genPanel.Layout()
         self.Layout()
         self.topBox.SetSizeHints(self)
@@ -350,56 +350,56 @@ ID_UNIFORM_NGROUPS = 4003
 ID_UNIFORM_STEP = 4004
 ID_UNIFORM_RETRIEVE = 4005
 
-class GenUniformPanel(wxPanel):
+class GenUniformPanel(wx.Panel):
 
     def __init__(self, parent, layer, fieldName, fieldType):
-        wxPanel.__init__(self, parent, -1)
+        wx.Panel.__init__(self, parent, -1)
 
         self.parent = parent
         self.layer = layer
         self.fieldName = fieldName
         self.fieldType = fieldType
 
-        topSizer = wxStaticBoxSizer(wxStaticBox(self, -1, ""), 
-                                    wxVERTICAL)
+        topSizer = wx.StaticBoxSizer(wx.StaticBox(self, -1, ""),
+                                    wx.VERTICAL)
 
         #############
 
-        sizer = wxBoxSizer(wxHORIZONTAL)
+        sizer = wx.BoxSizer(wx.HORIZONTAL)
 
-        sizer.Add(wxStaticText(self, -1, _("Min:")), 0, wxALL, 4) 
-        self.minCtrl = wxTextCtrl(self, ID_UNIFORM_MIN, style=wxTE_RIGHT)
-        sizer.Add(self.minCtrl, 1, wxALL, 4)
-        EVT_TEXT(self, ID_UNIFORM_MIN, self._OnRangeChanged)
+        sizer.Add(wx.StaticText(self, -1, _("Min:")), 0, wx.ALL, 4)
+        self.minCtrl = wx.TextCtrl(self, ID_UNIFORM_MIN, style=wx.TE_RIGHT)
+        sizer.Add(self.minCtrl, 1, wx.ALL, 4)
+        self.Bind(wx.EVT_TEXT, self._OnRangeChanged, id=ID_UNIFORM_MIN)
 
-        sizer.Add(wxStaticText(self, -1, _("Max:")), 0, wxALL, 4) 
-        self.maxCtrl = wxTextCtrl(self, ID_UNIFORM_MAX, style=wxTE_RIGHT)
-        sizer.Add(self.maxCtrl, 1, wxALL, 4)
-        EVT_TEXT(self, ID_UNIFORM_MAX, self._OnRangeChanged)
+        sizer.Add(wx.StaticText(self, -1, _("Max:")), 0, wx.ALL, 4)
+        self.maxCtrl = wx.TextCtrl(self, ID_UNIFORM_MAX, style=wx.TE_RIGHT)
+        sizer.Add(self.maxCtrl, 1, wx.ALL, 4)
+        self.Bind(wx.EVT_TEXT, self._OnRangeChanged, id=ID_UNIFORM_MAX)
 
-        sizer.Add(wxButton(self, ID_UNIFORM_RETRIEVE, _("Retrieve From Table")),
-            0, wxALL, 4)
-        EVT_BUTTON(self, ID_UNIFORM_RETRIEVE, self._OnRetrieve)
+        sizer.Add(wx.Button(self, ID_UNIFORM_RETRIEVE, _("Retrieve From Table")),
+            0, wx.ALL, 4)
+        self.Bind(wx.EVT_BUTTON, self._OnRetrieve, id=ID_UNIFORM_RETRIEVE)
 
-        topSizer.Add(sizer, 1, wxGROW, 0)
+        topSizer.Add(sizer, 1, wx.GROW, 0)
 
         #############
 
-        sizer = wxBoxSizer(wxHORIZONTAL)
+        sizer = wx.BoxSizer(wx.HORIZONTAL)
 
-        sizer.Add(wxStaticText(self, -1, _("Number of Groups:")), 0, wxALL, 4)
-        self.numGroupsCtrl = wxSpinCtrl(self, ID_UNIFORM_NGROUPS, 
-                                        style=wxTE_RIGHT)
-        EVT_TEXT(self, ID_UNIFORM_NGROUPS, self._OnNumGroupsChanged)
-        EVT_SPINCTRL(self, ID_UNIFORM_NGROUPS, self._OnNumGroupsChanged)
-        sizer.Add(self.numGroupsCtrl, 1, wxALL, 4)
+        sizer.Add(wx.StaticText(self, -1, _("Number of Groups:")), 0, wx.ALL, 4)
+        self.numGroupsCtrl = wx.SpinCtrl(self, ID_UNIFORM_NGROUPS,
+                                        style=wx.TE_RIGHT)
+        self.Bind(wx.EVT_TEXT, self._OnNumGroupsChanged, id=ID_UNIFORM_NGROUPS)
+        self.Bind(wx.EVT_SPINCTRL, self._OnNumGroupsChanged, id=ID_UNIFORM_NGROUPS)
+        sizer.Add(self.numGroupsCtrl, 1, wx.ALL, 4)
 
-        sizer.Add(wxStaticText(self, -1, _("Stepping:")), 0, wxALL, 4)
-        self.stepCtrl = wxTextCtrl(self, ID_UNIFORM_STEP, style=wxTE_RIGHT)
-        EVT_TEXT(self, ID_UNIFORM_STEP, self._OnSteppingChanged)
-        sizer.Add(self.stepCtrl , 1, wxALL, 4)
+        sizer.Add(wx.StaticText(self, -1, _("Stepping:")), 0, wx.ALL, 4)
+        self.stepCtrl = wx.TextCtrl(self, ID_UNIFORM_STEP, style=wx.TE_RIGHT)
+        self.Bind(wx.EVT_TEXT, self._OnSteppingChanged, id=ID_UNIFORM_STEP)
+        sizer.Add(self.stepCtrl , 1, wx.ALL, 4)
 
-        topSizer.Add(sizer, 1, wxGROW, 0)
+        topSizer.Add(sizer, 1, wx.GROW, 0)
 
         #############
 
@@ -420,14 +420,14 @@ class GenUniformPanel(wxPanel):
 
     def GetNumGroups(self):
         value = self.numGroupsCtrl.GetValue()
-        return self.__GetValidatedTypeEntry(self.numGroupsCtrl, 
+        return self.__GetValidatedTypeEntry(self.numGroupsCtrl,
                                             value,
                                             FIELDTYPE_INT,
                                             None)
 
     def GetStepping(self):
         step = self.stepCtrl.GetValue()
-        return self.__GetValidatedTypeEntry(self.stepCtrl, 
+        return self.__GetValidatedTypeEntry(self.stepCtrl,
                                             step,
                                             self.fieldType,
                                             0)
@@ -435,7 +435,7 @@ class GenUniformPanel(wxPanel):
     def GetMin(self):
         min = self.minCtrl.GetValue()
         max = self.maxCtrl.GetValue()
-        return self.__GetValidatedTypeEntry(self.minCtrl, 
+        return self.__GetValidatedTypeEntry(self.minCtrl,
                                             min,
                                             self.fieldType,
                                             max)
@@ -443,14 +443,14 @@ class GenUniformPanel(wxPanel):
     def GetMax(self):
         min = self.minCtrl.GetValue()
         max = self.maxCtrl.GetValue()
-        return self.__GetValidatedTypeEntry(self.maxCtrl, 
+        return self.__GetValidatedTypeEntry(self.maxCtrl,
                                             max,
                                             self.fieldType,
                                             min)
 
     def _OnRangeChanged(self, event):
 
-        hasFocus = wxWindow_FindFocus() == event.GetEventObject()
+        hasFocus = wx.Window_FindFocus() == event.GetEventObject()
         min = self.GetMin()
         max = self.GetMax()
 
@@ -548,7 +548,7 @@ class GenUniformPanel(wxPanel):
 
     def __GetValidatedTypeEntry(self, win, value, type, badValue = None):
 
-        if type == FIELDTYPE_INT: 
+        if type == FIELDTYPE_INT:
             func = int
         elif type == FIELDTYPE_DOUBLE:
             func = float
@@ -577,9 +577,9 @@ class GenUniformPanel(wxPanel):
             valid = False
 
         if valid:
-            win.SetForegroundColour(wxBLACK)
+            win.SetForegroundColour(wx.BLACK)
         else:
-            win.SetForegroundColour(wxRED)
+            win.SetForegroundColour(wx.RED)
 
         win.Refresh()
 
@@ -614,93 +614,93 @@ ID_UNIQUE_SORTUSE = 4007
 ID_UNIQUE_REVAVAIL = 4008
 ID_UNIQUE_REVUSE = 4009
 
-class GenUniquePanel(wxPanel):
+class GenUniquePanel(wx.Panel):
 
     def __init__(self, parent, layer, fieldName, fieldType):
-        wxPanel.__init__(self, parent, -1)
+        wx.Panel.__init__(self, parent, -1)
 
         self.parent = parent
         self.layer = layer
         self.fieldName = fieldName
         self.fieldType = fieldType
 
-        topSizer = wxStaticBoxSizer(wxStaticBox(self, -1, ""), 
-                                    wxVERTICAL)
+        topSizer = wx.StaticBoxSizer(wx.StaticBox(self, -1, ""),
+                                    wx.VERTICAL)
 
 
         #bsizer = wxBoxSizer(wxVERTICAL)
-        topSizer.Add(wxButton(self, ID_UNIQUE_RETRIEVE, 
-                            _("Retrieve From Table")), 
-                   0, wxALL | wxALIGN_RIGHT, 4)
+        topSizer.Add(wx.Button(self, ID_UNIQUE_RETRIEVE,
+                            _("Retrieve From Table")),
+                   0, wx.ALL | wx.ALIGN_RIGHT, 4)
 
-        EVT_BUTTON(self, ID_UNIQUE_RETRIEVE, self._OnRetrieve)
+        self.Bind(wx.EVT_BUTTON, self._OnRetrieve, id=ID_UNIQUE_RETRIEVE)
 
-        #topSizer.Add(bsizer, 0, wxALL, 4)
+        #topSizer.Add(bsizer, 0, wx.ALL, 4)
 
-        sizer = wxBoxSizer(wxHORIZONTAL)
+        sizer = wx.BoxSizer(wx.HORIZONTAL)
 
         self.dataList = []
 
-        psizer = wxBoxSizer(wxVERTICAL)
-        self.list_avail = wxListCtrl(self, -1, 
-                        style=wxLC_REPORT | wxLC_SINGLE_SEL)
+        psizer = wx.BoxSizer(wx.VERTICAL)
+        self.list_avail = wx.ListCtrl(self, -1,
+                        style=wx.LC_REPORT | wx.LC_SINGLE_SEL)
         self.list_avail.InsertColumn(0, _("Available"))
         self.list_avail_data = []
-        psizer.Add(self.list_avail, 1, wxGROW, 0)
+        psizer.Add(self.list_avail, 1, wx.GROW, 0)
 
-        bsizer = wxBoxSizer(wxHORIZONTAL)
-        bsizer.Add(wxButton(self, ID_UNIQUE_SORTAVAIL, _("Sort")))
-        EVT_BUTTON(self, ID_UNIQUE_SORTAVAIL, self._OnSortList)
+        bsizer = wx.BoxSizer(wx.HORIZONTAL)
+        bsizer.Add(wx.Button(self, ID_UNIQUE_SORTAVAIL, _("Sort")))
+        self.Bind(wx.EVT_BUTTON, self._OnSortList, id=ID_UNIQUE_SORTAVAIL)
 
-        bsizer.Add(wxButton(self, ID_UNIQUE_REVAVAIL, _("Reverse")))
-        EVT_BUTTON(self, ID_UNIQUE_REVAVAIL, self._OnReverseList)
+        bsizer.Add(wx.Button(self, ID_UNIQUE_REVAVAIL, _("Reverse")))
+        self.Bind(wx.EVT_BUTTON, self._OnReverseList, id=ID_UNIQUE_REVAVAIL)
 
-        psizer.Add(bsizer, 0, wxGROW, 0)
-        sizer.Add(psizer, 1, wxGROW, 0)
+        psizer.Add(bsizer, 0, wx.GROW, 0)
+        sizer.Add(psizer, 1, wx.GROW, 0)
 
-        
-        bsizer = wxBoxSizer(wxVERTICAL)
 
-        bmp = resource.GetBitmapResource(USEALL_BMP, wxBITMAP_TYPE_XPM)
-        bsizer.Add(wxBitmapButton(self, ID_UNIQUE_USEALL, bmp),
-                   0, wxGROW | wxALL, 4)
-        bmp = resource.GetBitmapResource(USE_BMP, wxBITMAP_TYPE_XPM)
-        bsizer.Add(wxBitmapButton(self, ID_UNIQUE_USE, bmp), 
-                   0, wxGROW | wxALL, 4)
-        bmp = resource.GetBitmapResource(USENOT_BMP, wxBITMAP_TYPE_XPM)
-        bsizer.Add(wxBitmapButton(self, ID_UNIQUE_DONTUSE, bmp), 
-                   0, wxGROW | wxALL, 4)
-        bmp = resource.GetBitmapResource(USENONE_BMP, wxBITMAP_TYPE_XPM)
-        bsizer.Add(wxBitmapButton(self, ID_UNIQUE_USENONE, bmp), 
-                   0, wxGROW | wxALL, 4)
+        bsizer = wx.BoxSizer(wx.VERTICAL)
 
-        EVT_BUTTON(self, ID_UNIQUE_USEALL, self._OnUseAll)
-        EVT_BUTTON(self, ID_UNIQUE_USE, self._OnUse)
-        EVT_BUTTON(self, ID_UNIQUE_DONTUSE, self._OnDontUse)
-        EVT_BUTTON(self, ID_UNIQUE_USENONE, self._OnUseNone)
+        bmp = resource.GetBitmapResource(USEALL_BMP, wx.BITMAP_TYPE_XPM)
+        bsizer.Add(wx.BitmapButton(self, ID_UNIQUE_USEALL, bmp),
+                   0, wx.GROW | wx.ALL, 4)
+        bmp = resource.GetBitmapResource(USE_BMP, wx.BITMAP_TYPE_XPM)
+        bsizer.Add(wx.BitmapButton(self, ID_UNIQUE_USE, bmp),
+                   0, wx.GROW | wx.ALL, 4)
+        bmp = resource.GetBitmapResource(USENOT_BMP, wx.BITMAP_TYPE_XPM)
+        bsizer.Add(wx.BitmapButton(self, ID_UNIQUE_DONTUSE, bmp),
+                   0, wx.GROW | wx.ALL, 4)
+        bmp = resource.GetBitmapResource(USENONE_BMP, wx.BITMAP_TYPE_XPM)
+        bsizer.Add(wx.BitmapButton(self, ID_UNIQUE_USENONE, bmp),
+                   0, wx.GROW | wx.ALL, 4)
 
-        sizer.Add(bsizer, 0, wxALL | wxALIGN_CENTER_VERTICAL, 4)
+        self.Bind(wx.EVT_BUTTON, self._OnUseAll, id=ID_UNIQUE_USEALL)
+        self.Bind(wx.EVT_BUTTON, self._OnUse, id=ID_UNIQUE_USE)
+        self.Bind(wx.EVT_BUTTON, self._OnDontUse, id=ID_UNIQUE_DONTUSE)
+        self.Bind(wx.EVT_BUTTON, self._OnUseNone, id=ID_UNIQUE_USENONE)
 
-        psizer = wxBoxSizer(wxVERTICAL)
-        self.list_use = wxListCtrl(self, -1, 
-                        style=wxLC_REPORT | wxLC_SINGLE_SEL)
+        sizer.Add(bsizer, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 4)
+
+        psizer = wx.BoxSizer(wx.VERTICAL)
+        self.list_use = wx.ListCtrl(self, -1,
+                        style=wx.LC_REPORT | wx.LC_SINGLE_SEL)
         self.list_use.InsertColumn(0, _("Use"))
         self.list_use_data = []
-        psizer.Add(self.list_use, 1, wxGROW, 0)
+        psizer.Add(self.list_use, 1, wx.GROW, 0)
 
-        bsizer = wxBoxSizer(wxHORIZONTAL)
-        bsizer.Add(wxButton(self, ID_UNIQUE_SORTUSE, _("Sort")))
-        EVT_BUTTON(self, ID_UNIQUE_SORTUSE, self._OnSortList)
+        bsizer = wx.BoxSizer(wx.HORIZONTAL)
+        bsizer.Add(wx.Button(self, ID_UNIQUE_SORTUSE, _("Sort")))
+        self.Bind(wx.EVT_BUTTON, self._OnSortList, id=ID_UNIQUE_SORTUSE)
 
-        bsizer.Add(wxButton(self, ID_UNIQUE_REVUSE, _("Reverse")))
-        EVT_BUTTON(self, ID_UNIQUE_REVUSE, self._OnReverseList)
+        bsizer.Add(wx.Button(self, ID_UNIQUE_REVUSE, _("Reverse")))
+        self.Bind(wx.EVT_BUTTON, self._OnReverseList, id=ID_UNIQUE_REVUSE)
 
-        psizer.Add(bsizer, 0, wxGROW, 0)
+        psizer.Add(bsizer, 0, wx.GROW, 0)
 
-        sizer.Add(psizer, 1, wxGROW, 0)
+        sizer.Add(psizer, 1, wx.GROW, 0)
 
 
-        topSizer.Add(sizer, 1, wxGROW, 0)
+        topSizer.Add(sizer, 1, wx.GROW, 0)
 
         self.SetSizer(topSizer)
         self.SetAutoLayout(True)
@@ -730,7 +730,7 @@ class GenUniquePanel(wxPanel):
         else:
             list = self.list_avail
 
-        list.SortItems(lambda i1, i2: cmp(self.dataList[i1], 
+        list.SortItems(lambda i1, i2: cmp(self.dataList[i1],
                                           self.dataList[i2]))
 
     def _OnReverseList(self, event):
@@ -760,7 +760,7 @@ class GenUniquePanel(wxPanel):
                 self.dataList.append(v)
                 i = self.list_avail.InsertStringItem(index, str(v))
                 self.list_avail.SetItemData(index, i)
-    
+
                 self.list_avail_data.append(v)
                 index += 1
         finally:
@@ -784,8 +784,8 @@ class GenUniquePanel(wxPanel):
     def __MoveSelectedItems(self, list_src, list_dest):
         while True:
             index = list_src.GetNextItem(-1,
-                                         wxLIST_NEXT_ALL, 
-                                         wxLIST_STATE_SELECTED)
+                                         wx.LIST_NEXT_ALL,
+                                         wx.LIST_STATE_SELECTED)
 
             if index == -1:
                 break
@@ -798,7 +798,7 @@ class GenUniquePanel(wxPanel):
         item = list_src.GetItem(index)
 
         x = list_dest.InsertStringItem(
-                list_dest.GetItemCount(), 
+                list_dest.GetItemCount(),
                 str(self.dataList[item.GetData()]))
 
         list_dest.SetItemData(x, item.GetData())
@@ -814,48 +814,48 @@ class GenUniquePanel(wxPanel):
 ID_QUANTILES_RANGE = 4001
 ID_QUANTILES_RETRIEVE = 4002
 
-class GenQuantilesPanel(wxPanel):
+class GenQuantilesPanel(wx.Panel):
 
     def __init__(self, parent, layer, fieldName, fieldType):
-        wxPanel.__init__(self, parent, -1)
+        wx.Panel.__init__(self, parent, -1)
 
         self.parent = parent
         self.layer = layer
         self.fieldName = fieldName
         self.fieldType = fieldType
 
-        topBox = wxStaticBoxSizer(wxStaticBox(self, -1, ""), 
-                                    wxVERTICAL)
+        topBox = wx.StaticBoxSizer(wx.StaticBox(self, -1, ""),
+                                    wx.VERTICAL)
 
-        self.text_range = wxTextCtrl(self, ID_QUANTILES_RANGE, "")
-        self.button_retrieve = wxButton(self, ID_QUANTILES_RETRIEVE,
+        self.text_range = wx.TextCtrl(self, ID_QUANTILES_RANGE, "")
+        self.button_retrieve = wx.Button(self, ID_QUANTILES_RETRIEVE,
                                         _("Retrieve from Table"))
 
-        self.spin_numClasses = wxSpinCtrl(self, -1, style=wxTE_RIGHT)
+        self.spin_numClasses = wx.SpinCtrl(self, -1, style=wx.TE_RIGHT)
         self.spin_numClasses.SetRange(2, sys.maxint)
         self.spin_numClasses.SetValue(2)
 
 
-        sizer = wxBoxSizer(wxHORIZONTAL)
-        sizer.Add(wxStaticText(self, -1, _("Apply to Range")), 0, wxALL, 4)
-        sizer.Add(self.text_range, 1, wxALL, 4)
-        sizer.Add(self.button_retrieve, 0, wxALL, 4)
+        sizer = wx.BoxSizer(wx.HORIZONTAL)
+        sizer.Add(wx.StaticText(self, -1, _("Apply to Range")), 0, wx.ALL, 4)
+        sizer.Add(self.text_range, 1, wx.ALL, 4)
+        sizer.Add(self.button_retrieve, 0, wx.ALL, 4)
 
-        topBox.Add(sizer, 0, wxEXPAND, 0)
+        topBox.Add(sizer, 0, wx.EXPAND, 0)
 
-        sizer = wxBoxSizer(wxHORIZONTAL)
-        sizer.Add(wxStaticText(self, -1, _("Number of Classes:")), 0, wxALL, 4)
-        sizer.Add(self.spin_numClasses, 1, wxALL, 4)
+        sizer = wx.BoxSizer(wx.HORIZONTAL)
+        sizer.Add(wx.StaticText(self, -1, _("Number of Classes:")), 0, wx.ALL, 4)
+        sizer.Add(self.spin_numClasses, 1, wx.ALL, 4)
 
-        topBox.Add(sizer, 0, wxEXPAND, 0)
+        topBox.Add(sizer, 0, wx.EXPAND, 0)
 
         self.SetSizer(topBox)
         self.SetAutoLayout(True)
         topBox.Fit(self)
         topBox.SetSizeHints(self)
 
-        EVT_TEXT(self, ID_QUANTILES_RANGE, self.OnRangeText)
-        EVT_BUTTON(self, ID_QUANTILES_RETRIEVE, self.OnRetrieve)
+        self.Bind(wx.EVT_TEXT, self.OnRangeText, id=ID_QUANTILES_RANGE)
+        self.Bind(wx.EVT_BUTTON, self.OnRetrieve, id=ID_QUANTILES_RETRIEVE)
 
         self.__range = None
 
@@ -893,9 +893,9 @@ class GenQuantilesPanel(wxPanel):
             self.__range = None
 
         if self.__range is not None:
-            self.text_range.SetForegroundColour(wxBLACK)
+            self.text_range.SetForegroundColour(wx.BLACK)
         else:
-            self.text_range.SetForegroundColour(wxRED)
+            self.text_range.SetForegroundColour(wx.RED)
 
     def OnRetrieve(self, event):
         table = self.layer.ShapeStore().Table()
@@ -904,6 +904,11 @@ class GenQuantilesPanel(wxPanel):
             try:
                 min, max = table.ValueRange(self.fieldName)
                 self.text_range.SetValue("[" + str(min) + ";" + str(max) + "]")
+                # This is a workaround, which will result in OnRangeText
+                # being called twice on some platforms. 
+                # Testing showed this is needed with current wx 2.4. versions 
+                # on MacOSX to guarantee that it is called at all.
+                self.OnRangeText(None)
             finally:
                 ThubanEndBusyCursor()
 
@@ -914,66 +919,66 @@ ID_CUSTOMRAMP_EDITEND = 4004
 ID_CUSTOMRAMP_SPROP = 4005
 ID_CUSTOMRAMP_EPROP = 4006
 
-class CustomRampPanel(wxPanel):
+class CustomRampPanel(wx.Panel):
 
     def __init__(self, parent, shapeType):
-        wxPanel.__init__(self, parent, -1)
+        wx.Panel.__init__(self, parent, -1)
 
-        topSizer = wxStaticBoxSizer(wxStaticBox(self, -1, ""), wxHORIZONTAL)
+        topSizer = wx.StaticBoxSizer(wx.StaticBox(self, -1, ""), wx.HORIZONTAL)
 
-        bsizer = wxBoxSizer(wxVERTICAL)
-        bsizer.Add(wxStaticText(self, -1, _("Start:")), 0, wxALL | wxCENTER, 4)
+        bsizer = wx.BoxSizer(wx.VERTICAL)
+        bsizer.Add(wx.StaticText(self, -1, _("Start:")), 0, wx.ALL | wx.CENTER, 4)
         self.startPropCtrl = classifier.ClassGroupPropertiesCtrl(
-            self, ID_CUSTOMRAMP_SPROP, 
+            self, ID_CUSTOMRAMP_SPROP,
             ClassGroupProperties(), shapeType,
-            style=wxSIMPLE_BORDER, size=(40, 20))
-        bsizer.Add(self.startPropCtrl, 1, wxGROW | wxALL | wxCENTER, 4)
-        bsizer.Add(wxButton(self, ID_CUSTOMRAMP_EDITSTART, _("Change")),
-                   0, wxGROW | wxALL | wxCENTER, 4)
+            style=wx.SIMPLE_BORDER, size=(40, 20))
+        bsizer.Add(self.startPropCtrl, 1, wx.GROW | wx.ALL | wx.CENTER, 4)
+        bsizer.Add(wx.Button(self, ID_CUSTOMRAMP_EDITSTART, _("Change")),
+                   0, wx.GROW | wx.ALL | wx.CENTER, 4)
 
         topSizer.Add(bsizer,
-                   1, wxALL \
-                      | wxSHAPED \
-                      | wxALIGN_CENTER_HORIZONTAL \
-                      | wxALIGN_CENTER_VERTICAL, \
+                   1, wx.ALL \
+                      | wx.SHAPED \
+                      | wx.ALIGN_CENTER_HORIZONTAL \
+                      | wx.ALIGN_CENTER_VERTICAL, \
                    4)
 
-        bmp = resource.GetBitmapResource(USE_BMP, wxBITMAP_TYPE_XPM)
-        bsizer = wxBoxSizer(wxVERTICAL)
-        bsizer.Add(wxBitmapButton(self, ID_CUSTOMRAMP_COPYSTART, bmp),
-                   0, wxGROW | wxALL, 4)
-        bmp = resource.GetBitmapResource(USENOT_BMP, wxBITMAP_TYPE_XPM)
-        bsizer.Add(wxBitmapButton(self, ID_CUSTOMRAMP_COPYEND, bmp),
-                   0, wxGROW | wxALL, 4)
+        bmp = resource.GetBitmapResource(USE_BMP, wx.BITMAP_TYPE_XPM)
+        bsizer = wx.BoxSizer(wx.VERTICAL)
+        bsizer.Add(wx.BitmapButton(self, ID_CUSTOMRAMP_COPYSTART, bmp),
+                   0, wx.GROW | wx.ALL, 4)
+        bmp = resource.GetBitmapResource(USENOT_BMP, wx.BITMAP_TYPE_XPM)
+        bsizer.Add(wx.BitmapButton(self, ID_CUSTOMRAMP_COPYEND, bmp),
+                   0, wx.GROW | wx.ALL, 4)
 
         topSizer.Add(bsizer,
-                   0, wxALL \
-                      | wxALIGN_CENTER_HORIZONTAL \
-                      | wxALIGN_CENTER_VERTICAL, 
+                   0, wx.ALL \
+                      | wx.ALIGN_CENTER_HORIZONTAL \
+                      | wx.ALIGN_CENTER_VERTICAL,
                    4)
 
-        bsizer = wxBoxSizer(wxVERTICAL)
-        bsizer.Add(wxStaticText(self, -1, _("End:")), 0, wxALL | wxCENTER, 4)
+        bsizer = wx.BoxSizer(wx.VERTICAL)
+        bsizer.Add(wx.StaticText(self, -1, _("End:")), 0, wx.ALL | wx.CENTER, 4)
         self.endPropCtrl = classifier.ClassGroupPropertiesCtrl(
-            self, ID_CUSTOMRAMP_EPROP, 
+            self, ID_CUSTOMRAMP_EPROP,
             ClassGroupProperties(), shapeType,
-            style=wxSIMPLE_BORDER, size=(40, 20))
-        bsizer.Add(self.endPropCtrl, 1, wxGROW | wxALL | wxCENTER, 4)
-        bsizer.Add(wxButton(self, ID_CUSTOMRAMP_EDITEND, _("Change")),
-                   0, wxGROW | wxALL | wxCENTER, 4)
+            style=wx.SIMPLE_BORDER, size=(40, 20))
+        bsizer.Add(self.endPropCtrl, 1, wx.GROW | wx.ALL | wx.CENTER, 4)
+        bsizer.Add(wx.Button(self, ID_CUSTOMRAMP_EDITEND, _("Change")),
+                   0, wx.GROW | wx.ALL | wx.CENTER, 4)
 
         topSizer.Add(bsizer,
-                   1, wxALL \
-                      | wxSHAPED \
-                      | wxALIGN_RIGHT \
-                      | wxALIGN_CENTER_HORIZONTAL \
-                      | wxALIGN_CENTER_VERTICAL, 
+                   1, wx.ALL \
+                      | wx.SHAPED \
+                      | wx.ALIGN_RIGHT \
+                      | wx.ALIGN_CENTER_HORIZONTAL \
+                      | wx.ALIGN_CENTER_VERTICAL,
                    4)
 
-        EVT_BUTTON(self, ID_CUSTOMRAMP_COPYSTART, self._OnCopyStart)
-        EVT_BUTTON(self, ID_CUSTOMRAMP_COPYEND, self._OnCopyEnd)
-        EVT_BUTTON(self, ID_CUSTOMRAMP_EDITSTART, self._OnEditStart)
-        EVT_BUTTON(self, ID_CUSTOMRAMP_EDITEND, self._OnEditEnd)
+        self.Bind(wx.EVT_BUTTON, self._OnCopyStart, id=ID_CUSTOMRAMP_COPYSTART)
+        self.Bind(wx.EVT_BUTTON, self._OnCopyEnd, id=ID_CUSTOMRAMP_COPYEND)
+        self.Bind(wx.EVT_BUTTON, self._OnEditStart, id=ID_CUSTOMRAMP_EDITSTART)
+        self.Bind(wx.EVT_BUTTON, self._OnEditEnd, id=ID_CUSTOMRAMP_EDITEND)
 
         self.SetSizer(topSizer)
         self.SetAutoLayout(True)
@@ -995,4 +1000,4 @@ class CustomRampPanel(wxPanel):
     def _OnEditEnd(self, event):
         self.endPropCtrl.DoEdit()
 
-  
+
