@@ -1,4 +1,4 @@
-# Copyright (c) 2001, 2002, 2003 by Intevation GmbH
+# Copyright (c) 2001, 2002, 2003, 2005 by Intevation GmbH
 # Authors:
 # Jonathan Coles <jonathan@intevation.de>
 # Frank Koormann <frank.koormann@intevation.de>
@@ -6,7 +6,7 @@
 # This program is free software under the GPL (>=v2)
 # Read the file COPYING coming with Thuban for details.
 
-__version__ = "$Revision: 1.36.2.1 $"
+__version__ = "$Revision: 2700 $"
 
 from  math import fabs, cos, pi
 
@@ -14,7 +14,7 @@ from Thuban import _
 
 import resource
 
-from wxPython.wx import *
+import wx
 
 from Thuban.Model.layer import BaseLayer
 from Thuban.Model.map import Map
@@ -31,6 +31,8 @@ from Thuban.UI.classifier import ClassDataPreviewer
 from Thuban.UI.dock import DockPanel
 from Thuban.UI.scalebar import ScaleBar
 
+from Thuban.UI.menu import Menu
+
 from Thuban.Lib.connector import ConnectorError
 
 ID_LEGEND_TOP = 4001
@@ -42,18 +44,8 @@ ID_LEGEND_PROPS = 4006
 ID_LEGEND_SHOWLAYER = 4007
 ID_LEGEND_HIDELAYER = 4008
 
-ID_POPUP_TOP = 4501
-ID_POPUP_UP  = 4502
-ID_POPUP_DOWN = 4503
-ID_POPUP_BOTTOM = 4504
-ID_POPUP_PROPS  = 4506
-ID_POPUP_VISIBLE = 4507
-ID_POPUP_PROJ = 4509
-ID_POPUP_REMOVE = 4510
-ID_POPUP_SHOWTABLE = 4511
-
-BMP_SIZE_W = 15
-BMP_SIZE_H = 15
+BMP_SIZE_W = 16
+BMP_SIZE_H = 16
 
 TOP_BMP = "top_layer"
 RAISE_BMP = "raise_layer"
@@ -63,10 +55,9 @@ SHOW_BMP  = "show_layer"
 HIDE_BMP  = "hide_layer"
 PROPS_BMP = "layer_properties"
 
-
 class LegendPanel(DockPanel):
 
-    def __init__(self, parent, map, mainWindow): 
+    def __init__(self, parent, map, mainWindow):
         DockPanel.__init__(self, parent, -1)
 
         self.mainWindow = mainWindow
@@ -74,66 +65,56 @@ class LegendPanel(DockPanel):
 
         self.buttons = []
 
-        panelBox = wxBoxSizer(wxVERTICAL)
+        panelBox = wx.BoxSizer(wx.VERTICAL)
 
-        self.toolBar = wxToolBar(self, -1)
-        self.toolBar.SetToolBitmapSize(wxSize(24, 24))
+        self.toolBar = wx.ToolBar(self, -1)
+        self.toolBar.SetToolBitmapSize(wx.Size(24, 24))
 
-        bmp = resource.GetBitmapResource(TOP_BMP, wxBITMAP_TYPE_XPM)
-        self.toolBar.AddTool(ID_LEGEND_TOP, bmp, 
+        bmp = resource.GetBitmapResource(TOP_BMP, wx.BITMAP_TYPE_XPM)
+        self.toolBar.AddTool(ID_LEGEND_TOP, bmp,
             shortHelpString=_("Top Layer"))
 
-        bmp = resource.GetBitmapResource(RAISE_BMP, wxBITMAP_TYPE_XPM)
-        self.toolBar.AddTool(ID_LEGEND_RAISE, bmp, 
+        bmp = resource.GetBitmapResource(RAISE_BMP, wx.BITMAP_TYPE_XPM)
+        self.toolBar.AddTool(ID_LEGEND_RAISE, bmp,
             shortHelpString=_("Raise Layer"))
 
-        bmp = resource.GetBitmapResource(LOWER_BMP, wxBITMAP_TYPE_XPM)
-        self.toolBar.AddTool(ID_LEGEND_LOWER, bmp, 
+        bmp = resource.GetBitmapResource(LOWER_BMP, wx.BITMAP_TYPE_XPM)
+        self.toolBar.AddTool(ID_LEGEND_LOWER, bmp,
             shortHelpString=_("Lower Layer"))
 
-        bmp = resource.GetBitmapResource(BOTTOM_BMP, wxBITMAP_TYPE_XPM)
-        self.toolBar.AddTool(ID_LEGEND_BOTTOM, bmp, 
+        bmp = resource.GetBitmapResource(BOTTOM_BMP, wx.BITMAP_TYPE_XPM)
+        self.toolBar.AddTool(ID_LEGEND_BOTTOM, bmp,
             shortHelpString=_("Bottom Layer"))
 
-        bmp = resource.GetBitmapResource(SHOW_BMP, wxBITMAP_TYPE_XPM)
-        self.toolBar.AddTool(ID_LEGEND_SHOWLAYER, bmp, 
+        bmp = resource.GetBitmapResource(SHOW_BMP, wx.BITMAP_TYPE_XPM)
+        self.toolBar.AddTool(ID_LEGEND_SHOWLAYER, bmp,
             shortHelpString=_("Show Layer"))
 
-        bmp = resource.GetBitmapResource(HIDE_BMP, wxBITMAP_TYPE_XPM)
-        self.toolBar.AddTool(ID_LEGEND_HIDELAYER, bmp, 
+        bmp = resource.GetBitmapResource(HIDE_BMP, wx.BITMAP_TYPE_XPM)
+        self.toolBar.AddTool(ID_LEGEND_HIDELAYER, bmp,
             shortHelpString=_("Hide Layer"))
 
-        bmp = resource.GetBitmapResource(PROPS_BMP, wxBITMAP_TYPE_XPM)
-        self.toolBar.AddTool(ID_LEGEND_PROPS, bmp, 
+        bmp = resource.GetBitmapResource(PROPS_BMP, wx.BITMAP_TYPE_XPM)
+        self.toolBar.AddTool(ID_LEGEND_PROPS, bmp,
             shortHelpString=_("Edit Layer Properties"))
 
         self.toolBar.Realize()
-        panelBox.Add(self.toolBar, 0, wxGROW, 0)
+        panelBox.Add(self.toolBar, 0, wx.GROW, 0)
 
-        EVT_TOOL(self, ID_LEGEND_TOP, self._OnMoveTop)
-        EVT_TOOL(self, ID_LEGEND_RAISE, self._OnMoveUp)
-        EVT_TOOL(self, ID_LEGEND_LOWER, self._OnMoveDown)
-        EVT_TOOL(self, ID_LEGEND_BOTTOM, self._OnMoveBottom)
-        EVT_TOOL(self, ID_LEGEND_PROPS, self._OnProperties)
-        EVT_TOOL(self, ID_LEGEND_SHOWLAYER, self._OnShowLayer)
-        EVT_TOOL(self, ID_LEGEND_HIDELAYER, self._OnHideLayer)
+        self.Bind(wx.EVT_TOOL, self._OnMoveTop, id=ID_LEGEND_TOP)
+        self.Bind(wx.EVT_TOOL, self._OnMoveUp, id=ID_LEGEND_RAISE)
+        self.Bind(wx.EVT_TOOL, self._OnMoveDown, id=ID_LEGEND_LOWER)
+        self.Bind(wx.EVT_TOOL, self._OnMoveBottom, id=ID_LEGEND_BOTTOM)
+        self.Bind(wx.EVT_TOOL, self._OnProperties, id=ID_LEGEND_PROPS)
+        self.Bind(wx.EVT_TOOL, self._OnShowLayer, id=ID_LEGEND_SHOWLAYER)
+        self.Bind(wx.EVT_TOOL, self._OnHideLayer, id=ID_LEGEND_HIDELAYER)
 
-        EVT_MENU(self, ID_POPUP_PROPS, self._OnProperties)
-        EVT_MENU(self, ID_POPUP_TOP, self._OnMoveTop)
-        EVT_MENU(self, ID_POPUP_UP, self._OnMoveUp)
-        EVT_MENU(self, ID_POPUP_DOWN, self._OnMoveDown)
-        EVT_MENU(self, ID_POPUP_BOTTOM, self._OnMoveBottom)
-        EVT_MENU(self, ID_POPUP_VISIBLE, self._OnToggleVisibility)
-        EVT_MENU(self, ID_POPUP_PROJ, self._OnProjection)
-        EVT_MENU(self, ID_POPUP_REMOVE, self._OnRemoveLayer)
-        EVT_MENU(self, ID_POPUP_SHOWTABLE, self._OnShowTable)
-        
         self.tree = LegendTree(self, ID_LEGEND_TREE, map, mainWindow)
 
-        panelBox.Add(self.tree, 1, wxGROW, 0)
+        panelBox.Add(self.tree, 1, wx.GROW, 0)
 
-        self.scalebarbitmap = ScaleBarBitmap(self, map, mainWindow) 
-        panelBox.Add(self.scalebarbitmap, 0, wxGROW, 0)
+        self.scalebarbitmap = ScaleBarBitmap(self, map, mainWindow)
+        panelBox.Add(self.scalebarbitmap, 0, wx.GROW, 0)
 
         self.SetAutoLayout(True)
         self.SetSizer(panelBox)
@@ -146,7 +127,7 @@ class LegendPanel(DockPanel):
 
         self.Create()
 
-        EVT_CLOSE(self, self._OnClose)
+        self.Bind(wx.EVT_CLOSE, self._OnClose)
 
 
     def GetMap(self):
@@ -178,8 +159,8 @@ class LegendPanel(DockPanel):
 
     def _OnMoveTop(self, event):
         self.tree.MoveCurrentItemTop()
-        
-    def _OnMoveUp(self, event): 
+
+    def _OnMoveUp(self, event):
         self.tree.MoveCurrentItemUp()
 
     def _OnMoveDown(self, event):
@@ -194,7 +175,7 @@ class LegendPanel(DockPanel):
 
     #def Close(self, force = False):
         #DockPanel.Close(self, force)
-        
+
     def _OnClose(self, event):
         self.__Close()
 
@@ -226,11 +207,11 @@ class LegendPanel(DockPanel):
     def __Close(self):
         self.tree.Close()
 
-class LegendTree(wxTreeCtrl):
+class LegendTree(wx.TreeCtrl):
 
     def __init__(self, parent, id, map, mainWindow):
-        wxTreeCtrl.__init__(self, parent, id,
-                            style = wxTR_DEFAULT_STYLE | wxTR_HIDE_ROOT,
+        wx.TreeCtrl.__init__(self, parent, id,
+                            style = wx.TR_DEFAULT_STYLE | wx.TR_HIDE_ROOT,
                             size = (200, 200))
 
         self.mainWindow = mainWindow
@@ -258,13 +239,13 @@ class LegendTree(wxTreeCtrl):
         self.preventExpandCollapse = False
         self.raiseProperties = False
 
-        EVT_TREE_ITEM_ACTIVATED(self, ID_LEGEND_TREE, self._OnItemActivated)
-        EVT_TREE_SEL_CHANGED(self, ID_LEGEND_TREE, self._OnSelChanged)
-        EVT_TREE_ITEM_EXPANDING(self, ID_LEGEND_TREE, self.OnItemExpandCollapse)
-        EVT_TREE_ITEM_COLLAPSING(self, ID_LEGEND_TREE, self.OnItemExpandCollapse)
-        EVT_TREE_ITEM_RIGHT_CLICK(self, ID_LEGEND_TREE, self._OnRightClick)
+        self.Bind(wx.EVT_TREE_ITEM_ACTIVATED, self._OnItemActivated, id=ID_LEGEND_TREE)
+        self.Bind(wx.EVT_TREE_SEL_CHANGED, self._OnSelChanged, id=ID_LEGEND_TREE)
+        self.Bind(wx.EVT_TREE_ITEM_EXPANDING, self.OnItemExpandCollapse, id=ID_LEGEND_TREE)
+        self.Bind(wx.EVT_TREE_ITEM_COLLAPSING, self.OnItemExpandCollapse, id=ID_LEGEND_TREE)
+        self.Bind(wx.EVT_TREE_ITEM_RIGHT_CLICK, self._OnRightClick, id=ID_LEGEND_TREE)
 
-        EVT_CLOSE(self, self._OnClose)
+        self.Bind(wx.EVT_CLOSE, self._OnClose)
 
         self.SetMap(map)
 
@@ -280,42 +261,30 @@ class LegendTree(wxTreeCtrl):
         item = event.GetItem()
         self.SelectItem(item)
 
-        # Create the menu
-        menu = wxMenu("", 0)
-
-        # The "Visible" item is a special ...
-        menuitem = wxMenuItem(menu, ID_POPUP_VISIBLE, _("Visible"), 
-                                "", wxITEM_CHECK)
-        menu.AppendItem(menuitem)
-        layer, group = self.GetSelectedHierarchy()
-        menuitem.Check(layer.Visible())
-
-        menu.AppendSeparator()
-        menu.Append(ID_POPUP_PROPS, _("&Properties..."))
-        menu.Append(ID_POPUP_PROJ, _("Pro&jection..."))
-        menu.Append(ID_POPUP_REMOVE, _("&Remove Layer"))
-        menu.Append(ID_POPUP_SHOWTABLE, _("Show Ta&ble"))
-
-        # grey out this Show Table item if the Layer has no table to show
-        menuitem = menu.FindItemById(ID_POPUP_SHOWTABLE)
-        if not layer.HasShapes():
-            menuitem.Enable(False)
-
-        menu.AppendSeparator()
-        menu.Append(ID_POPUP_TOP, _("Top Layer"))
-        menu.Append(ID_POPUP_UP, _("Raise Layer"))
-        menu.Append(ID_POPUP_DOWN, _("Lower Layer"))
-        menu.Append(ID_POPUP_BOTTOM, _("Bottom Layer"))
+        # Define the menu
+        popup_menu = Menu("PopUp", "",
+                          [ "layer_visibility",
+                            None,
+                            "layer_properties",
+                            "layer_projection",
+                            "layer_remove",
+                            "layer_show_table",
+                            None,
+                            "layer_to_top",
+                            "layer_raise",
+                            "layer_lower",
+                            "layer_to_bottom"
+                            ])
 
         # Display the menu
         pos = event.GetPoint()
         shift = self.ClientToScreen((0,0))
-        self.PopupMenu(menu, pos)
+        self.PopupMenu(self.mainWindow.build_menu(popup_menu), pos)
 
     def find_layer(self, layer):
         """Return the tree item for the layer"""
         root = self.GetRootItem()
-        id, cookie = self.GetFirstChild(root, 0)
+        id, cookie = self.GetFirstChild(root)
         while id.IsOk():
             if self.GetPyData(id) is layer:
                 return id
@@ -344,7 +313,7 @@ class LegendTree(wxTreeCtrl):
             #except ConnectorError:
                 #pass
             self.DeleteAllItems()
-        
+
         self.map = map
 
         if self.map is not None:
@@ -400,7 +369,7 @@ class LegendTree(wxTreeCtrl):
             layers = self.map.Layers()
             return layers.index(data2) - layers.index(data1)
         else:
-            return wxTreeCtrl.OnCompareItems(self, item1, item2)
+            return wx.TreeCtrl.OnCompareItems(self, item1, item2)
 
     def DoOnShowLayer(self):
         layer, group = self.GetSelectedHierarchy()
@@ -441,7 +410,7 @@ class LegendTree(wxTreeCtrl):
     def _OnMsgMapsChanged(self):
         #print self.map is self.mainWindow.Map()
         self.SetMap(self.mainWindow.Map())
-        
+
     def _OnSelChanged(self, event):
         # If we change the selection from normalize_selection do nothing.
         if self.changing_selection:
@@ -518,7 +487,7 @@ class LegendTree(wxTreeCtrl):
         # Build a dict with all layers known by the the tree as keys
         layers = {}
         root = self.GetRootItem()
-        id, cookie = self.GetFirstChild(root, 0)
+        id, cookie = self.GetFirstChild(root)
         while id.IsOk():
             layers[self.GetPyData(id)] = 1
             id, cookie = self.GetNextChild(root, cookie)
@@ -537,7 +506,7 @@ class LegendTree(wxTreeCtrl):
         layers = map.Layers()
 
         root = self.GetRootItem()
-        id, cookie = self.GetFirstChild(root, 0)
+        id, cookie = self.GetFirstChild(root)
         while id.IsOk():
             if self.GetPyData(id) not in layers:
                 self.__RemoveLayer(id)
@@ -562,7 +531,7 @@ class LegendTree(wxTreeCtrl):
     def __UpdateSelection(self):
         layer, group = self.GetSelectedHierarchy()
         self.parent.DoOnSelChanged(layer, group)
-        
+
     def __FillTree(self, map):
 
         self.Freeze()
@@ -577,6 +546,7 @@ class LegendTree(wxTreeCtrl):
         self.Thaw()
 
     def __FillTreeLayer(self, pid):
+
         layer = self.GetPyData(pid)
 
         self.Freeze()
@@ -599,8 +569,9 @@ class LegendTree(wxTreeCtrl):
                     bmp = self.__BuildGroupImage(g, shapeType)
 
                     if bmp is None:
-                        self.SetItemImage(id, -1)
-                        self.SetItemSelectedImage(id, -1)
+                        self.SetItemImage(id, -1, wx.TreeItemIcon_Normal)
+                        self.SetItemImage(id, -1, wx.TreeItemIcon_Selected)
+                        #self.SetItemSelectedImage(id, -1)
                     else:
                         if self.availImgListIndices:
                             i = self.availImgListIndices.pop(0)
@@ -608,16 +579,17 @@ class LegendTree(wxTreeCtrl):
                         else:
                             i = self.image_list.Add(bmp)
 
-                        self.SetItemImage(id, i)
-                        self.SetItemSelectedImage(id, i)
+                        self.SetItemImage(id, i, wx.TreeItemIcon_Normal)
+                        self.SetItemImage(id, i, wx.TreeItemIcon_Selected)
+                        #self.SetItemlectedImage(id, i)
 
         self.Thaw()
 
     def __BuildGroupImage(self, group, shapeType):
 
-        bmp = wxEmptyBitmap(BMP_SIZE_W, BMP_SIZE_H)
+        bmp = wx.EmptyBitmap(BMP_SIZE_W, BMP_SIZE_H)
         #brush = wxBrush(Color2wxColour(item[1]), wxSOLID)
-        dc = wxMemoryDC()
+        dc = wx.MemoryDC()
         dc.SelectObject(bmp)
         dc.Clear()
 
@@ -629,17 +601,17 @@ class LegendTree(wxTreeCtrl):
 
         pid = self.GetRootItem()
 
-        id, cookie = self.GetFirstChild(pid, 123)
+        id, cookie = self.GetFirstChild(pid)
         while id.IsOk():
             self.__RemoveLayer(id)
             id, cookie = self.GetNextChild(pid, cookie)
 
-        wxTreeCtrl.DeleteAllItems(self)
+        wx.TreeCtrl.DeleteAllItems(self)
 
     def __AddLayer(self, before, l):
         root = self.GetRootItem()
         id = self.InsertItemBefore(root, before,
-                            l.Title(), 
+                            l.Title(),
                             self.mapImageIndex,
                             self.mapImageIndex)
 
@@ -650,7 +622,7 @@ class LegendTree(wxTreeCtrl):
         self.Expand(id)
 
         l.Subscribe(LAYER_CHANGED, self._OnMsgLayerChanged)
-        l.Subscribe(LAYER_VISIBILITY_CHANGED, 
+        l.Subscribe(LAYER_VISIBILITY_CHANGED,
                     self._OnMsgLayerVisibilityChanged)
         l.Subscribe(TITLE_CHANGED, self._OnMsgLayerTitleChanged)
 
@@ -658,39 +630,39 @@ class LegendTree(wxTreeCtrl):
         self.DeleteChildren(id)
 
         layer = self.GetPyData(id)
-        layer.Unsubscribe(LAYER_CHANGED, 
+        layer.Unsubscribe(LAYER_CHANGED,
                           self._OnMsgLayerChanged)
-        layer.Unsubscribe(LAYER_VISIBILITY_CHANGED, 
+        layer.Unsubscribe(LAYER_VISIBILITY_CHANGED,
                           self._OnMsgLayerVisibilityChanged)
         layer.Unsubscribe(TITLE_CHANGED, self._OnMsgLayerTitleChanged)
 
         self.Delete(id)
 
     def DeleteChildren(self, pid):
-        id, cookie = self.GetFirstChild(pid, 123)
+        id, cookie = self.GetFirstChild(pid)
         while id.IsOk():
             self.availImgListIndices.append(self.GetItemImage(id))
             id, cookie = self.GetNextChild(pid, cookie)
-        wxTreeCtrl.DeleteChildren(self, pid)
+        wx.TreeCtrl.DeleteChildren(self, pid)
 
     def GetRootItem(self):
-        root = wxTreeCtrl.GetRootItem(self)
+        root = wx.TreeCtrl.GetRootItem(self)
 
         if not root.IsOk():
-            self.image_list = wxImageList(BMP_SIZE_W, BMP_SIZE_H, False, 0)
+            self.image_list = wx.ImageList(BMP_SIZE_W, BMP_SIZE_H, False, 0)
 
-            bmp = wxEmptyBitmap(BMP_SIZE_W, BMP_SIZE_H)
-            dc = wxMemoryDC()
+            bmp = wx.EmptyBitmap(BMP_SIZE_W, BMP_SIZE_H)
+            dc = wx.MemoryDC()
             dc.SelectObject(bmp)
-            dc.SetBrush(wxBLACK_BRUSH)
+            dc.SetBrush(wx.BLACK_BRUSH)
             dc.Clear()
-            dc.SelectObject(wxNullBitmap)
+            dc.SelectObject(wx.NullBitmap)
 
             self.emptyImageIndex = \
-                self.image_list.AddWithColourMask(bmp, wxColour(0, 0, 0))
+                self.image_list.AddWithColourMask(bmp, wx.Colour(0, 0, 0))
 
-            bmp = resource.GetBitmapResource("legend_icon_layer", 
-                                              wxBITMAP_TYPE_XPM)
+            bmp = resource.GetBitmapResource("legend_icon_layer",
+                                              wx.BITMAP_TYPE_XPM)
             self.mapImageIndex = \
                 self.image_list.Add(bmp)
 
@@ -705,16 +677,16 @@ class LegendTree(wxTreeCtrl):
         font = self.GetItemFont(id)
 
         if visible:
-            font.SetStyle(wxNORMAL)
-            color = wxBLACK
+            font.SetStyle(wx.NORMAL)
+            color = wx.BLACK
         else:
             #font.SetStyle(wxITALIC)
-            font.SetStyle(wxNORMAL)
-            color = wxLIGHT_GREY
+            font.SetStyle(wx.NORMAL)
+            color = wx.LIGHT_GREY
 
         self.SetItemTextColour(id, color)
         self.SetItemFont(id, font)
-        
+
     def __ShowHideLayer(self, layer):
         parent = self.find_layer(layer)
         assert parent.IsOk()
@@ -723,25 +695,38 @@ class LegendTree(wxTreeCtrl):
 
         self.__SetVisibilityStyle(visible, parent)
 
-        id, cookie = self.GetFirstChild(parent, 123)
+        id, cookie = self.GetFirstChild(parent)
 
         while id.IsOk():
             self.__SetVisibilityStyle(visible, id)
             id, cookie = self.GetNextChild(parent, cookie)
-            
-class ScaleBarBitmap(wxBoxSizer):
+
+    # In wxPython 2.4 the GetFirstChild method has to be called with a
+    # second argument and in 2.5 it must not.  Reading the code of
+    # wxPython 2.4 it seems that the second parameter was intended to be
+    # optional there but due to a bug in the C++ code it doesn't work
+    # and omitting the second argument leads to a segfault.  To cope
+    # with this and to make the code usable with both 2.5 and 2.4 we
+    # overwrite the inherited method when running with 2.4 to provide a
+    # default value for the second argument.
+    if map(int, wx.__version__.split(".")[:2]) < [2, 5]:
+        def GetFirstChild(self, item):
+            return wx.TreeCtrl.GetFirstChild(self, item, 0)
+
+
+class ScaleBarBitmap(wx.BoxSizer):
 
     def __init__(self, parent, map, mainWindow):
         # While the width is fixed, get the height _now_.
-        dc = wxMemoryDC()
+        dc = wx.MemoryDC()
         textwidth, textheight = dc.GetTextExtent("%d"%0)
         self.width = 210
         self.height = textheight + 3*2 + 8
 
-        wxBoxSizer.__init__(self, wxVERTICAL)
-        bmp=wxEmptyBitmap(self.width, self.height)
-        self.scalebarBitmap = wxStaticBitmap(parent, -1, bmp)
-        self.Add(self.scalebarBitmap, 0, wxALIGN_CENTER|wxLEFT|wxTOP|wxRIGHT, 1)
+        wx.BoxSizer.__init__(self, wx.VERTICAL)
+        bmp=wx.EmptyBitmap(self.width, self.height)
+        self.scalebarBitmap = wx.StaticBitmap(parent, -1, bmp)
+        self.Add(self.scalebarBitmap, 0, wx.ALIGN_CENTER|wx.LEFT|wx.TOP|wx.RIGHT, 1)
 
         self.mainWindow = mainWindow
         self.parent = parent
@@ -753,7 +738,7 @@ class ScaleBarBitmap(wxBoxSizer):
 
         if self.canvas is not None:
             for msg, func in sub_list: self.canvas.Unsubscribe(msg, func)
-        
+
         self.canvas = canvas
         self.scalebar = ScaleBar(canvas.map)
 
@@ -765,8 +750,8 @@ class ScaleBarBitmap(wxBoxSizer):
         self.__SetScale(scale)
 
     def __SetScale(self, scale):
-        bmp = wxEmptyBitmap(self.width, self.height)
-        dc = wxMemoryDC()
+        bmp = wx.EmptyBitmap(self.width, self.height)
+        dc = wx.MemoryDC()
         dc.SelectObject(bmp)
         dc.Clear()
 
