@@ -1,4 +1,4 @@
-# Copyright (C) 2001, 2002, 2003, 2004 by Intevation GmbH
+# Copyright (C) 2001, 2002, 2003, 2004, 2005 by Intevation GmbH
 # Authors:
 # Jan-Oliver Wagner <jan@intevation.de>
 # Bernhard Herzog <bh@intevation.de>
@@ -11,31 +11,29 @@
 The main window
 """
 
-__version__ = "$Revision: 1.128 $"
-# $Source: /thubanrepository/thuban/Thuban/UI/mainwindow.py,v $
-# $Id: mainwindow.py,v 1.128 2004/02/13 10:46:44 bh Exp $
+__version__ = "$Revision: 2700 $"
+# $Source$
+# $Id: mainwindow.py 2700 2006-09-18 14:27:02Z dpinte $
 
 import os
 import copy
 
-from wxPython.wx import *
+import wx
 
 import Thuban
 
 from Thuban import _
-from Thuban.Model.messages import TITLE_CHANGED
+from Thuban.Model.messages import TITLE_CHANGED, LAYER_PROJECTION_CHANGED, \
+     MAP_PROJECTION_CHANGED, MAP_LAYERS_ADDED, MAP_LAYERS_REMOVED
+
 from Thuban.Model.session import create_empty_session
 from Thuban.Model.layer import Layer, RasterLayer
 from Thuban.Model.postgisdb import PostGISShapeStore, has_postgis_support
-# XXX: replace this by
-# from wxPython.lib.dialogs import wxMultipleChoiceDialog
-# when Thuban does not support wxPython 2.4.0 any more.
-from Thuban.UI.multiplechoicedialog import wxMultipleChoiceDialog
+from wx.lib.dialogs import MultipleChoiceDialog
 
 import view
 import tree
 import tableview, identifyview
-from Thuban.UI.classifier import Classifier
 import legend
 from menu import Menu
 
@@ -53,6 +51,15 @@ import Thuban.Model.resource
 
 import projdialog
 
+from Thuban.UI.classifier import Classifier
+from Thuban.UI.rasterlayerproperties import RasterLayerProperties
+from Thuban.Model.layer import RasterLayer
+
+from Thuban.Lib.classmapper import ClassMapper
+
+layer_properties_dialogs = ClassMapper()
+layer_properties_dialogs.add(RasterLayer, RasterLayerProperties)
+layer_properties_dialogs.add(Layer, Classifier)
 
 class MainWindow(DockFrame):
 
@@ -73,9 +80,15 @@ class MainWindow(DockFrame):
                          "SelectedShapes": "canvas",
                          }
 
+    # Messages from the canvas that may require a status bar update.
+    # The update_status_bar method will be subscribed to these messages.
+    update_status_bar_messages = (VIEW_POSITION, LAYER_PROJECTION_CHANGED,
+                                  MAP_PROJECTION_CHANGED, MAP_LAYERS_ADDED,
+                                  MAP_LAYERS_REMOVED)
+
     def __init__(self, parent, ID, title, application, interactor,
-                 initial_message = None, size = wxSize(-1, -1)):
-        DockFrame.__init__(self, parent, ID, title, wxDefaultPosition, size)
+                 initial_message = None, size = wx.Size(-1, -1)):
+        DockFrame.__init__(self, parent, ID, title, wx.DefaultPosition, size)
         #wxFrame.__init__(self, parent, ID, title, wxDefaultPosition, size)
 
         self.application = application
@@ -99,11 +112,13 @@ class MainWindow(DockFrame):
 
         # Create the map canvas
         canvas = view.MapCanvas(self, -1)
-        canvas.Subscribe(VIEW_POSITION, self.view_position_changed)
         canvas.Subscribe(SHAPES_SELECTED, self.identify_view_on_demand)
         self.canvas = canvas
         self.canvas.Subscribe(TITLE_CHANGED, self.title_changed)
 
+        for channel in self.update_status_bar_messages:
+            self.canvas.Subscribe(channel, self.update_status_bar)
+
         self.SetMainWindow(self.canvas)
 
         self.SetAutoLayout(True)
@@ -112,7 +127,7 @@ class MainWindow(DockFrame):
 
         self.ShowLegend()
 
-        EVT_CLOSE(self, self.OnClose)
+        self.Bind(wx.EVT_CLOSE, self.OnClose)
 
     def Subscribe(self, channel, *args):
         """Subscribe a function to a message channel.
@@ -136,7 +151,7 @@ class MainWindow(DockFrame):
             object = getattr(self, self.delegated_messages[channel])
             try:
                 object.Unsubscribe(channel, *args)
-            except wxPyDeadObjectError:
+            except wx.PyDeadObjectError:
                 # The object was a wxObject and has already been
                 # destroyed. Hopefully it has unsubscribed all its
                 # subscribers already so that it's OK if we do nothing
@@ -175,13 +190,13 @@ class MainWindow(DockFrame):
         """Bind the necessary events for the given command and ID"""
         if not self.events_bound.has_key(ID):
             # the events haven't been bound yet
-            EVT_MENU(self, ID, self.invoke_command)
+            self.Bind(wx.EVT_MENU, self.invoke_command, id=ID)
             if command.IsDynamic():
-                EVT_UPDATE_UI(self, ID, self.update_command_ui)
+                self.Bind(wx.EVT_UPDATE_UI, self.update_command_ui, id=ID)
 
     def build_menu_bar(self, menudesc):
         """Build and return the menu bar from the menu description"""
-        menu_bar = wxMenuBar()
+        menu_bar = wx.MenuBar()
 
         for item in menudesc.items:
             # here the items must all be Menu instances themselves
@@ -191,7 +206,7 @@ class MainWindow(DockFrame):
 
     def build_menu(self, menudesc):
         """Return a wxMenu built from the menu description menudesc"""
-        wxmenu = wxMenu()
+        wxmenu = wx.Menu()
         last = None
         for item in menudesc.items:
             if item is None:
@@ -201,7 +216,7 @@ class MainWindow(DockFrame):
                     wxmenu.AppendSeparator()
             elif isinstance(item, Menu):
                 # a submenu
-                wxmenu.AppendMenu(wxNewId(), item.title, self.build_menu(item))
+                wxmenu.AppendMenu(wx.NewId(), item.title, self.build_menu(item))
             else:
                 # must the name the name of a command
                 self.add_menu_command(wxmenu, item)
@@ -214,13 +229,13 @@ class MainWindow(DockFrame):
         The parameter should be an instance of the Menu class but it
         should not contain submenus.
         """
-        toolbar = self.CreateToolBar(wxTB_3DBUTTONS)
+        toolbar = self.CreateToolBar(wx.TB_3DBUTTONS)
 
         # set the size of the tools' bitmaps. Not needed on wxGTK, but
         # on Windows, although it doesn't work very well there. It seems
         # that only 16x16 icons are really supported on windows.
         # We probably shouldn't hardwire the bitmap size here.
-        toolbar.SetToolBitmapSize(wxSize(24, 24))
+        toolbar.SetToolBitmapSize(wx.Size(24, 24))
 
         for item in toolbardesc.items:
             if item is None:
@@ -261,8 +276,8 @@ class MainWindow(DockFrame):
             command = registry.Command(name)
             if command is not None:
                 ID = self.get_id(name)
-                bitmap = resource.GetBitmapResource(command.Icon(), 
-                                                    wxBITMAP_TYPE_XPM)
+                bitmap = resource.GetBitmapResource(command.Icon(),
+                                                    wx.BITMAP_TYPE_XPM)
                 toolbar.AddTool(ID, bitmap,
                                 shortHelpString = command.HelpText(),
                                 isToggle = command.IsCheckCommand())
@@ -302,10 +317,10 @@ class MainWindow(DockFrame):
             if command.IsCheckCommand():
                     event.Check(command.Checked(context))
 
-    def RunMessageBox(self, title, text, flags = wxOK | wxICON_INFORMATION):
+    def RunMessageBox(self, title, text, flags = wx.OK | wx.ICON_INFORMATION):
         """Run a modal message box with the given text, title and flags
         and return the result"""
-        dlg = wxMessageDialog(self, text, title, flags)
+        dlg = wx.MessageDialog(self, text, title, flags)
         dlg.CenterOnParent()
         result = dlg.ShowModal()
         dlg.Destroy()
@@ -332,23 +347,92 @@ class MainWindow(DockFrame):
     def get_open_dialog(self, name):
         return self.dialogs.get(name)
 
-    def view_position_changed(self):
+    def update_status_bar(self, *args):
+        """Handler for a bunch of messages that may require a status bar update
+
+        Currently this handles the canvas' VIEW_POSITION_CHANGED
+        messages as well as several messages about changes in the map
+        which may affect whether and how projections are used.
+
+        These messages affect the text in the status bar in the following way:
+
+        When VIEW_POSITION_CHANGED messages are sent and the mouse is
+        actually in the canvas window, display the current mouse
+        coordinates as defined by the canvas' CurrentPosition method.
+
+        If there is no current position to show, check whether there is
+        a potential problem with the map and layer projections and
+        display a message about it.  Otherwise the status bar will
+        become empty.
+
+        The text is displayed in the status bar using the
+        set_position_text method.
+        """
+        # Implementation note: We do not really have to know which
+        # message was sent.  We can simply call the canvas'
+        # CurrentPosition method and if that returns a tuple, it was a
+        # VIEW_POSITION_CHANGED message and we have to display it.
+        # Otherwise it was a VIEW_POSITION_CHANGED message where the
+        # mouse has left the canvas or it was a message about a change
+        # to the map, in which case we check the projections.
+        #
+        # When changing this method, keep in mind that the
+        # VIEW_POSITION_CHANGED message are sent for every mouse move in
+        # the canvas window, that is they happen very often, so the path
+        # taken in that case has to be fast.
+        text = ""
         pos = self.canvas.CurrentPosition()
         if pos is not None:
             text = "(%10.10g, %10.10g)" % pos
         else:
-            text = ""
+            for layer in self.canvas.Map().Layers():
+                bbox = layer.LatLongBoundingBox()
+                if bbox:
+                    left, bottom, right, top = bbox
+                    if not (-180 <= left <= 180 and
+                            -180 <= right <= 180 and
+                            -90 <= top <= 90 and
+                            -90 <= bottom <= 90):
+                        text = _("Select layer '%s' and pick a projection "
+                                 "using Layer/Projection...") % layer.title
+                        break
+
         self.set_position_text(text)
 
     def set_position_text(self, text):
-        """Set the statusbar text showing the current position.
+        """Set the statusbar text to that created by update_status_bar
 
         By default the text is shown in field 0 of the status bar.
         Override this method in derived classes to put it into a
         different field of the statusbar.
+
+        For historical reasons this method is called set_position_text
+        because at first the text was always either the current position
+        or the empty string.  Now it can contain other messages as well.
+        The method will be renamed at one point.
         """
+        # Note: If this method is renamed we should perhaps think about
+        # some backwards compatibility measures for projects like
+        # GREAT-ER which override this method.
         self.SetStatusText(text)
 
+    def OpenOrRaiseDialog(self, name, dialog_class, *args, **kw):
+        """
+        Open or raise a dialog.
+
+        If a dialog with the denoted name does already exist it is
+        raised.  Otherwise a new dialog, an instance of dialog_class,
+        is created, inserted into the main list and displayed.
+        """
+        dialog = self.get_open_dialog(name)
+
+        if dialog is None:
+            dialog = dialog_class(self, name, *args, **kw)
+            self.add_dialog(name, dialog)
+            dialog.Show(True)
+        else:
+            dialog.Raise()
+
     def save_modified_session(self, can_veto = 1):
         """If the current session has been modified, ask the user
         whether to save it and do so if requested. Return the outcome of
@@ -359,30 +443,30 @@ class MainWindow(DockFrame):
         a cancel button, otherwise not.
         """
         if self.application.session.WasModified():
-            flags = wxYES_NO | wxICON_QUESTION
+            flags = wx.YES_NO | wx.ICON_QUESTION
             if can_veto:
-                flags = flags | wxCANCEL
+                flags = flags | wx.CANCEL
             result = self.RunMessageBox(_("Exit"),
                                         _("The session has been modified."
                                          " Do you want to save it?"),
                                         flags)
-            if result == wxID_YES:
+            if result == wx.ID_YES:
                 self.SaveSession()
         else:
-            result = wxID_NO
+            result = wx.ID_NO
         return result
 
     def NewSession(self):
-        if self.save_modified_session() != wxID_CANCEL:
+        if self.save_modified_session() != wx.ID_CANCEL:
             self.application.SetSession(create_empty_session())
 
     def OpenSession(self):
-        if self.save_modified_session() != wxID_CANCEL:
-            dlg = wxFileDialog(self, _("Open Session"),
-                               self.application.Path("data"), "", 
-                               "Thuban Session File (*.thuban)|*.thuban", 
-                               wxOPEN)
-            if dlg.ShowModal() == wxID_OK:
+        if self.save_modified_session() != wx.ID_CANCEL:
+            dlg = wx.FileDialog(self, _("Open Session"),
+                               self.application.Path("data"), "",
+                               "Thuban Session File (*.thuban)|*.thuban",
+                               wx.OPEN)
+            if dlg.ShowModal() == wx.ID_OK:
                 self.application.OpenSession(dlg.GetPath(),
                                              self.run_db_param_dialog)
                 self.application.SetPath("data", dlg.GetPath())
@@ -400,11 +484,11 @@ class MainWindow(DockFrame):
             self.application.SaveSession()
 
     def SaveSessionAs(self):
-        dlg = wxFileDialog(self, _("Save Session As"),
+        dlg = wx.FileDialog(self, _("Save Session As"),
                            self.application.Path("data"), "",
-                           "Thuban Session File (*.thuban)|*.thuban", 
-                           wxSAVE|wxOVERWRITE_PROMPT)
-        if dlg.ShowModal() == wxID_OK:
+                           "Thuban Session File (*.thuban)|*.thuban",
+                           wx.SAVE|wx.OVERWRITE_PROMPT)
+        if dlg.ShowModal() == wx.ID_OK:
             self.application.session.SetFilename(dlg.GetPath())
             self.application.SaveSession()
             self.application.SetPath("data",dlg.GetPath())
@@ -415,13 +499,15 @@ class MainWindow(DockFrame):
 
     def OnClose(self, event):
         result = self.save_modified_session(can_veto = event.CanVeto())
-        if result == wxID_CANCEL:
+        if result == wx.ID_CANCEL:
             event.Veto()
         else:
             # FIXME: it would be better to tie the unsubscription to
             # wx's destroy event, but that isn't implemented for wxGTK
             # yet.
-            self.canvas.Unsubscribe(VIEW_POSITION, self.view_position_changed)
+            for channel in self.update_status_bar_messages:
+                self.canvas.Unsubscribe(channel, self.update_status_bar)
+
             DockFrame.OnClose(self, event)
             for dlg in self.dialogs.values():
                 dlg.Destroy()
@@ -472,12 +558,12 @@ class MainWindow(DockFrame):
         dialog.Raise()
 
     def AddLayer(self):
-        dlg = wxFileDialog(self, _("Select one or more data files"), 
+        dlg = wx.FileDialog(self, _("Select one or more data files"),
                            self.application.Path("data"), "",
                            _("Shapefiles (*.shp)") + "|*.shp;*.SHP|" +
                            _("All Files (*.*)") + "|*.*",
-                           wxOPEN | wxMULTIPLE)
-        if dlg.ShowModal() == wxID_OK:
+                           wx.OPEN | wx.MULTIPLE)
+        if dlg.ShowModal() == wx.ID_OK:
             filenames = dlg.GetPaths()
             for filename in filenames:
                 title = os.path.splitext(os.path.basename(filename))[0]
@@ -500,10 +586,10 @@ class MainWindow(DockFrame):
         dlg.Destroy()
 
     def AddRasterLayer(self):
-        dlg = wxFileDialog(self, _("Select an image file"), 
+        dlg = wx.FileDialog(self, _("Select an image file"),
                            self.application.Path("data"), "", "*.*",
-                           wxOPEN)
-        if dlg.ShowModal() == wxID_OK:
+                           wx.OPEN)
+        if dlg.ShowModal() == wx.ID_OK:
             filename = dlg.GetPath()
             title = os.path.splitext(os.path.basename(filename))[0]
             map = self.canvas.Map()
@@ -528,14 +614,15 @@ class MainWindow(DockFrame):
         session = self.application.Session()
         dlg = ChooseDBTableDialog(self, self.application.Session())
 
-        if dlg.ShowModal() == wxID_OK:
-            dbconn, dbtable = dlg.GetTable()
+        if dlg.ShowModal() == wx.ID_OK:
+            dbconn, dbtable, id_column, geo_column = dlg.GetTable()
             try:
                 title = str(dbtable)
 
                 # Chose the correct Interface for the database type
-                store = PostGISShapeStore(dbconn, dbtable)
-                session.AddShapeStore(store)
+                store = session.OpenDBShapeStore(dbconn, dbtable,
+                                                 id_column = id_column,
+                                                 geometry_column = geo_column)
                 layer = Layer(title, store)
             except:
                 # Some error occured while initializing the layer
@@ -571,6 +658,11 @@ class MainWindow(DockFrame):
             return self.canvas.Map().CanRemoveLayer(layer)
         return False
 
+    def LayerToTop(self):
+        layer = self.current_layer()
+        if layer is not None:
+            self.canvas.Map().MoveLayerToTop(layer)
+
     def RaiseLayer(self):
         layer = self.current_layer()
         if layer is not None:
@@ -581,6 +673,11 @@ class MainWindow(DockFrame):
         if layer is not None:
             self.canvas.Map().LowerLayer(layer)
 
+    def LayerToBottom(self):
+        layer = self.current_layer()
+        if layer is not None:
+            self.canvas.Map().MoveLayerToBottom(layer)
+
     def current_layer(self):
         """Return the currently selected layer.
 
@@ -603,12 +700,16 @@ class MainWindow(DockFrame):
     def HideLayer(self):
         layer = self.current_layer()
         if layer is not None:
-            layer.SetVisible(0)
+            layer.SetVisible(False)
 
     def ShowLayer(self):
         layer = self.current_layer()
         if layer is not None:
-            layer.SetVisible(1)
+            layer.SetVisible(True)
+
+    def ToggleLayerVisibility(self):
+        layer = self.current_layer()
+        layer.SetVisible(not layer.Visible())
 
     def DuplicateLayer(self):
         """Ceate a new layer above the selected layer with the same shapestore
@@ -619,6 +720,8 @@ class MainWindow(DockFrame):
                               layer.ShapeStore(),
                               projection = layer.GetProjection())
             new_classification = copy.deepcopy(layer.GetClassification())
+            new_layer.SetClassificationColumn(
+                    layer.GetClassificationColumn())
             new_layer.SetClassification(new_classification)
             self.Map().AddLayer(new_layer)
 
@@ -628,8 +731,16 @@ class MainWindow(DockFrame):
         return layer is not None and hasattr(layer, "ShapeStore")
 
     def LayerShowTable(self):
+        """
+        Present a TableView Window for the current layer.
+        In case the window is already open, bring it to the front.
+        In case, there is no active layer, do nothing.
+        In case, the layer has no ShapeStore, do nothing.
+        """
         layer = self.current_layer()
         if layer is not None:
+            if not hasattr(layer, "ShapeStore"):
+                return
             table = layer.ShapeStore().Table()
             name = "table_view" + str(id(table))
             dialog = self.get_open_dialog(name)
@@ -640,8 +751,7 @@ class MainWindow(DockFrame):
                 self.add_dialog(name, dialog)
                 dialog.Show(True)
             else:
-                # FIXME: bring dialog to front here
-                pass
+                dialog.Raise()
 
     def MapProjection(self):
 
@@ -650,7 +760,7 @@ class MainWindow(DockFrame):
 
         if dialog is None:
             map = self.canvas.Map()
-            dialog = projdialog.ProjFrame(self, name, 
+            dialog = projdialog.ProjFrame(self, name,
                      _("Map Projection: %s") % map.Title(), map)
             self.add_dialog(name, dialog)
             dialog.Show()
@@ -665,7 +775,7 @@ class MainWindow(DockFrame):
 
         if dialog is None:
             map = self.canvas.Map()
-            dialog = projdialog.ProjFrame(self, name, 
+            dialog = projdialog.ProjFrame(self, name,
                      _("Layer Projection: %s") % layer.Title(), layer)
             self.add_dialog(name, dialog)
             dialog.Show()
@@ -683,14 +793,18 @@ class MainWindow(DockFrame):
         self.OpenLayerProperties(layer)
 
     def OpenLayerProperties(self, layer, group = None):
-        name = "layer_properties" + str(id(layer))
-        dialog = self.get_open_dialog(name)
+        """
+        Open or raise the properties dialog.
 
-        if dialog is None:
-            dialog = Classifier(self, name, self.Map(), layer, group)
-            self.add_dialog(name, dialog)
-            dialog.Show()
-        dialog.Raise()
+        This method opens or raises the properties dialog for the
+        currently selected layer if one is defined for this layer
+        type.
+        """
+        dialog_class = layer_properties_dialogs.get(layer)
+
+        if dialog_class is not None:
+            name = "layer_properties" + str(id(layer))
+            self.OpenOrRaiseDialog(name, dialog_class, layer, group = group)
 
     def LayerJoinTable(self):
         layer = self.canvas.SelectedLayer()
@@ -717,7 +831,7 @@ class MainWindow(DockFrame):
         dialog = self.FindRegisteredDock(name)
 
         if dialog is None:
-            dialog = self.CreateDock(name, -1, _("Legend"), wxLAYOUT_LEFT)
+            dialog = self.CreateDock(name, -1, _("Legend"), wx.LAYOUT_LEFT)
             legend.LegendPanel(dialog, None, self)
             dialog.Dock()
             dialog.GetPanel().SetMap(self.Map())
@@ -727,17 +841,17 @@ class MainWindow(DockFrame):
 
     def LegendShown(self):
         """Return true iff the legend is currently open"""
-        dialog = self.FindRegisteredDock("legend") 
+        dialog = self.FindRegisteredDock("legend")
         return dialog is not None and dialog.IsShown()
 
     def TableOpen(self):
-        dlg = wxFileDialog(self, _("Open Table"), 
-                           self.application.Path("data"), "", 
-                           _("DBF Files (*.dbf)") + "|*.dbf|" + 
+        dlg = wx.FileDialog(self, _("Open Table"),
+                           self.application.Path("data"), "",
+                           _("DBF Files (*.dbf)") + "|*.dbf|" +
                            #_("CSV Files (*.csv)") + "|*.csv|" + 
-                           _("All Files (*.*)") + "|*.*", 
-                           wxOPEN)
-        if dlg.ShowModal() == wxID_OK:
+                           _("All Files (*.*)") + "|*.*",
+                           wx.OPEN)
+        if dlg.ShowModal() == wx.ID_OK:
             filename = dlg.GetPath()
             dlg.Destroy()
             try:
@@ -756,12 +870,12 @@ class MainWindow(DockFrame):
         lst = [(t.Title(), t) for t in tables]
         lst.sort()
         titles = [i[0] for i in lst]
-        dlg = wxMultipleChoiceDialog(self, _("Pick the tables to close:"),
+        dlg = MultipleChoiceDialog(self, _("Pick the tables to close:"),
                                      _("Close Table"), titles,
                                      size = (400, 300),
-                                     style = wxDEFAULT_DIALOG_STYLE | 
-                                             wxRESIZE_BORDER)
-        if dlg.ShowModal() == wxID_OK:
+                                     style = wx.DEFAULT_DIALOG_STYLE |
+                                             wx.RESIZE_BORDER)
+        if dlg.ShowModal() == wx.ID_OK:
             for i in dlg.GetValue():
                 self.application.session.RemoveTable(lst[i][1])
 
@@ -777,12 +891,12 @@ class MainWindow(DockFrame):
         lst = [(t.Title(), t) for t in tables]
         lst.sort()
         titles = [i[0] for i in lst]
-        dlg = wxMultipleChoiceDialog(self, _("Pick the table to show:"),
+        dlg = MultipleChoiceDialog(self, _("Pick the table to show:"),
                                      _("Show Table"), titles,
-                                     size = (400,300), 
-                                     style = wxDEFAULT_DIALOG_STYLE | 
-                                             wxRESIZE_BORDER)
-        if (dlg.ShowModal() == wxID_OK):
+                                     size = (400,300),
+                                     style = wx.DEFAULT_DIALOG_STYLE |
+                                             wx.RESIZE_BORDER)
+        if (dlg.ShowModal() == wx.ID_OK):
             for i in dlg.GetValue():
                 # XXX: if the table belongs to a layer, open a
                 # LayerTableFrame instead of QueryTableFrame
@@ -812,12 +926,12 @@ class MainWindow(DockFrame):
         lst = [(t.Title(), t) for t in tables]
         lst.sort()
         titles = [i[0] for i in lst]
-        dlg = wxMultipleChoiceDialog(self, _("Pick the table to rename:"),
+        dlg = MultipleChoiceDialog(self, _("Pick the table to rename:"),
                                      _("Rename Table"), titles,
-                                     size = (400,300), 
-                                     style = wxDEFAULT_DIALOG_STYLE | 
-                                             wxRESIZE_BORDER)
-        if (dlg.ShowModal() == wxID_OK):
+                                     size = (400,300),
+                                     style = wx.DEFAULT_DIALOG_STYLE |
+                                             wx.RESIZE_BORDER)
+        if (dlg.ShowModal() == wx.ID_OK):
             to_rename = [lst[i][1] for i in dlg.GetValue()]
             dlg.Destroy()
         else:
@@ -825,10 +939,10 @@ class MainWindow(DockFrame):
 
         # Second, let the user rename the layers
         for table in to_rename:
-            dlg = wxTextEntryDialog(self, _("Table Title:"), _("Rename Table"),
+            dlg = wx.TextEntryDialog(self, _("Table Title:"), _("Rename Table"),
                                     table.Title())
             try:
-                if dlg.ShowModal() == wxID_OK:
+                if dlg.ShowModal() == wx.ID_OK:
                     title = dlg.GetValue()
                     if title != "":
                         table.SetTitle(title)
@@ -874,9 +988,9 @@ class MainWindow(DockFrame):
         self.canvas.Print()
 
     def RenameMap(self):
-        dlg = wxTextEntryDialog(self, _("Map Title:"), _("Rename Map"),
+        dlg = wx.TextEntryDialog(self, _("Map Title:"), _("Rename Map"),
                                 self.Map().Title())
-        if dlg.ShowModal() == wxID_OK:
+        if dlg.ShowModal() == wx.ID_OK:
             title = dlg.GetValue()
             if title != "":
                 self.Map().SetTitle(title)
@@ -887,10 +1001,10 @@ class MainWindow(DockFrame):
         """Let the user rename the currently selected layer"""
         layer = self.current_layer()
         if layer is not None:
-            dlg = wxTextEntryDialog(self, _("Layer Title:"), _("Rename Layer"),
+            dlg = wx.TextEntryDialog(self, _("Layer Title:"), _("Rename Layer"),
                                     layer.Title())
             try:
-                if dlg.ShowModal() == wxID_OK:
+                if dlg.ShowModal() == wx.ID_OK:
                     title = dlg.GetValue()
                     if title != "":
                         layer.SetTitle(title)
@@ -986,6 +1100,14 @@ def _has_selected_layer(context):
     """Return true if a layer is selected in the context"""
     return context.mainwindow.has_selected_layer()
 
+def _has_selected_layer_visible(context):
+    """Return true if a layer is selected in the context which is
+    visible."""
+    if context.mainwindow.has_selected_layer():
+        layer = context.mainwindow.current_layer()
+        if layer.Visible(): return True
+    return False
+
 def _has_selected_shape_layer(context):
     """Return true if a shape layer is selected in the context"""
     return context.mainwindow.has_selected_shape_layer()
@@ -1009,8 +1131,8 @@ def _has_visible_map(context):
     if map is not None:
         for layer in map.Layers():
             if layer.Visible():
-                return 1
-    return 0
+                return True
+    return False
 
 def _has_legend_shown(context):
     """Return true if the legend window is shown"""
@@ -1134,6 +1256,18 @@ _method_command("layer_jointable", _("&Join Table..."), "LayerJoinTable",
                 sensitive = _has_selected_shape_layer,
                 helptext = _("Join and attach a table to the selected layer"))
 
+# further layer methods:
+_method_command("layer_to_top", _("&Top"), "LayerToTop",
+                helptext = _("Put selected layer to the top"),
+                sensitive = _has_selected_layer)
+_method_command("layer_to_bottom", _("&Bottom"), "LayerToBottom",
+                helptext = _("Put selected layer to the bottom"),
+                sensitive = _has_selected_layer)
+_method_command("layer_visibility", _("&Visible"), "ToggleLayerVisibility",
+                checked = _has_selected_layer_visible,
+                helptext = _("Toggle visibility of selected layer"),
+                sensitive = _has_selected_layer)
+
 def _can_unjoin(context):
     """Return whether the Layer/Unjoin command can be executed.
 
@@ -1188,7 +1322,7 @@ map_menu = ["layer_add", "layer_add_db", "rasterlayer_add", "layer_remove",
                         None,
                         "toggle_legend",
                         None]
-if wxPlatform == '__WXMSW__':
+if wx.Platform == '__WXMSW__':
     map_menu.append("map_export")
 map_menu.append("map_print")
 
@@ -1227,10 +1361,10 @@ main_menu = Menu("<main>", "<main>",
 # the main toolbar
 
 main_toolbar = Menu("<toolbar>", "<toolbar>",
-                    ["map_zoom_in_tool", "map_zoom_out_tool", "map_pan_tool", 
-                     "map_full_extent", 
-                     "layer_full_extent", 
-                     "selected_full_extent", 
-                     None, 
+                    ["map_zoom_in_tool", "map_zoom_out_tool", "map_pan_tool",
+                     "map_full_extent",
+                     "layer_full_extent",
+                     "selected_full_extent",
+                     None,
                      "map_identify_tool", "map_label_tool"])
 
