@@ -7,26 +7,26 @@
 
 """Acess to UI-related resources"""
 
-__version__ = "$Revision: 1.3 $"
-# $Source: /thubanrepository/thuban/Thuban/UI/resource.py,v $
-# $Id: resource.py,v 1.3 2003/08/01 16:13:49 bh Exp $
+__version__ = "$Revision: 2700 $"
+# $Source$
+# $Id: resource.py 2700 2006-09-18 14:27:02Z dpinte $
 
 
 import os
 import Thuban
 
-from wxPython.wx import wxBITMAP_TYPE_XPM, wxBITMAP_TYPE_ANY, wxBitmap, wxImage
+import wx
 
 
 bitmapdir = os.path.join(Thuban.__path__[0], os.pardir, "Resources", "Bitmaps")
-bitmap_extensions = {wxBITMAP_TYPE_XPM: ".xpm",
-                     wxBITMAP_TYPE_ANY: ""}
+bitmap_extensions = {wx.BITMAP_TYPE_XPM: ".xpm",
+                     wx.BITMAP_TYPE_ANY: ""}
 
 def GetBitmapResource(file, type):
     filename = os.path.join(bitmapdir, file) + bitmap_extensions[type]
-    return wxBitmap(filename, type)
+    return wx.Bitmap(filename, type)
 
 def GetImageResource(file, type):
     filename = os.path.join(bitmapdir, file) + bitmap_extensions[type]
-    return wxImage(filename, type)
+    return wx.Image(filename, type)
 
