@@ -5,14 +5,14 @@
 # This program is free software under the GPL (>=v2)
 # Read the file COPYING coming with Thuban for details.
 
-__version__ = "$Revision: 1.34 $"
+__version__ = "$Revision: 2718 $"
 
 import os.path
 
 from Thuban import _
 
-from wxPython.wx import *
-from wxPython.grid import *
+import wx
+from wx import grid
 
 from Thuban.Lib.connector import Publisher
 from Thuban.Model.table import FIELDTYPE_INT, FIELDTYPE_DOUBLE, \
@@ -23,20 +23,20 @@ from messages import SHAPES_SELECTED, SESSION_REPLACED
 from Thuban.Model.messages import TABLE_REMOVED, MAP_LAYERS_REMOVED
 from Thuban.UI.common import ThubanBeginBusyCursor, ThubanEndBusyCursor
 
-wx_value_type_map = {FIELDTYPE_INT: wxGRID_VALUE_NUMBER,
-                     FIELDTYPE_DOUBLE: wxGRID_VALUE_FLOAT,
-                     FIELDTYPE_STRING: wxGRID_VALUE_STRING}
+wx_value_type_map = {FIELDTYPE_INT: grid.GRID_VALUE_NUMBER,
+                     FIELDTYPE_DOUBLE: grid.GRID_VALUE_FLOAT,
+                     FIELDTYPE_STRING: grid.GRID_VALUE_STRING}
 
 ROW_SELECTED = "ROW_SELECTED"
 
 QUERY_KEY = 'S'
 
-class DataTable(wxPyGridTableBase):
+class DataTable(grid.PyGridTableBase):
 
     """Wrapper around a Thuban table object suitable for a wxGrid"""
 
     def __init__(self, table = None):
-        wxPyGridTableBase.__init__(self)
+        grid.PyGridTableBase.__init__(self)
         self.num_cols = 0
         self.num_rows = 0
         self.columns = []
@@ -71,7 +71,16 @@ class DataTable(wxPyGridTableBase):
     # Renderer understands the type too,) not just strings as in the
     # C++ version.
     def GetValue(self, row, col):
-        record = self.table.ReadRowAsDict(row, row_is_ordinal = 1)
+        try:
+            record = self.table.ReadRowAsDict(row, row_is_ordinal = 1)
+        except UnicodeError:
+            record = dict()
+            for (key, val) in self.table.ReadRowAsDict(row, \
+                              row_is_ordinal = 1).items():
+                if isinstance(val, str):
+                    record[key] = val.decode('iso-8859-1')
+                else:
+                    record[key] = val
         return record[self.columns[col][0]]
 
     def SetValue(self, row, col, value):
@@ -112,7 +121,7 @@ class DataTable(wxPyGridTableBase):
         return self.table.RowOrdinalToId(ordinal)
 
 
-class NullRenderer(wxPyGridCellRenderer):
+class NullRenderer(grid.PyGridCellRenderer):
 
     """Renderer that draws NULL as a gray rectangle
 
@@ -121,15 +130,15 @@ class NullRenderer(wxPyGridCellRenderer):
     """
 
     def __init__(self, non_null_renderer):
-        wxPyGridCellRenderer.__init__(self)
+        grid.PyGridCellRenderer.__init__(self)
         self.non_null_renderer = non_null_renderer
 
     def Draw(self, grid, attr, dc, rect, row, col, isSelected):
         value = grid.table.GetValue(row, col)
         if value is None:
-            dc.SetBackgroundMode(wxSOLID)
-            dc.SetBrush(wxBrush(wxColour(192, 192, 192), wxSOLID))
-            dc.SetPen(wxTRANSPARENT_PEN)
+            dc.SetBackgroundMode(wx.SOLID)
+            dc.SetBrush(wx.Brush(wx.Colour(192, 192, 192), wx.SOLID))
+            dc.SetPen(wx.TRANSPARENT_PEN)
             dc.DrawRectangle(rect.x, rect.y, rect.width, rect.height)
         else:
             self.non_null_renderer.Draw(grid, attr, dc, rect, row, col,
@@ -142,7 +151,7 @@ class NullRenderer(wxPyGridCellRenderer):
         return NullRenderer(self.non_null_renderer)
 
 
-class TableGrid(wxGrid, Publisher):
+class TableGrid(grid.Grid, Publisher):
 
     """A grid view for a Thuban table
 
@@ -155,7 +164,7 @@ class TableGrid(wxGrid, Publisher):
     """
 
     def __init__(self, parent, table = None):
-        wxGrid.__init__(self, parent, -1)
+        grid.Grid.__init__(self, parent, -1)
 
         self.allow_messages_count = 0
 
@@ -178,22 +187,22 @@ class TableGrid(wxGrid, Publisher):
         # long time.
         #self.AutoSizeColumns(False)
 
-        self.SetSelectionMode(wxGrid.wxGridSelectRows)
+        self.SetSelectionMode(grid.Grid.wxGridSelectRows)
 
         self.ToggleEventListeners(True)
-        EVT_GRID_RANGE_SELECT(self, self.OnRangeSelect)
-        EVT_GRID_SELECT_CELL(self, self.OnSelectCell)
+        self.Bind(grid.EVT_GRID_RANGE_SELECT, self.OnRangeSelect)
+        self.Bind(grid.EVT_GRID_SELECT_CELL, self.OnSelectCell)
 
         # Replace the normal renderers with our own versions which
         # render NULL/None values specially
-        self.RegisterDataType(wxGRID_VALUE_STRING,
-                              NullRenderer(wxGridCellStringRenderer()), None)
-        self.RegisterDataType(wxGRID_VALUE_NUMBER,
-                              NullRenderer(wxGridCellNumberRenderer()), None)
-        self.RegisterDataType(wxGRID_VALUE_FLOAT,
-                              NullRenderer(wxGridCellFloatRenderer()), None)
+        self.RegisterDataType(grid.GRID_VALUE_STRING,
+                              NullRenderer(grid.GridCellStringRenderer()), None)
+        self.RegisterDataType(grid.GRID_VALUE_NUMBER,
+                              NullRenderer(grid.GridCellNumberRenderer()), None)
+        self.RegisterDataType(grid.GRID_VALUE_FLOAT,
+                              NullRenderer(grid.GridCellFloatRenderer()), None)
 
-        EVT_WINDOW_DESTROY(self, self.OnDestroy)
+        self.Bind(wx.EVT_WINDOW_DESTROY, self.OnDestroy)
 
     def OnDestroy(self, event):
         Publisher.Destroy(self)
@@ -306,7 +315,7 @@ class TableFrame(ThubanFrame):
 
     def __init__(self, parent, name, title, table):
         ThubanFrame.__init__(self, parent, name, title)
-        self.panel = wxPanel(self, -1)
+        self.panel = wx.Panel(self, -1)
 
         self.table = table
         self.grid = self.make_grid(self.table)
@@ -364,19 +373,19 @@ class QueryTableFrame(TableFrame):
     def __init__(self, parent, name, title, table):
         TableFrame.__init__(self, parent, name, title, table)
 
-        self.combo_fields = wxComboBox(self.panel, -1, style=wxCB_READONLY)
-        self.choice_comp = wxChoice(self.panel, -1, 
+        self.combo_fields = wx.ComboBox(self.panel, -1, style=wx.CB_READONLY)
+        self.choice_comp = wx.Choice(self.panel, -1,
                               choices=["<", "<=", "==", "!=", ">=", ">"])
-        self.combo_value = wxComboBox(self.panel, ID_COMBOVALUE)
-        self.choice_action = wxChoice(self.panel, -1, 
-                                choices=[_("Replace Selection"), 
-                                        _("Refine Selection"), 
+        self.combo_value = wx.ComboBox(self.panel, ID_COMBOVALUE)
+        self.choice_action = wx.Choice(self.panel, -1,
+                                choices=[_("Replace Selection"),
+                                        _("Refine Selection"),
                                         _("Add to Selection")])
 
-        button_query = wxButton(self.panel, ID_QUERY, _("Query"))
-        button_export = wxButton(self.panel, ID_EXPORT, _("Export"))
-        button_exportSel = wxButton(self.panel, ID_EXPORTSEL, _("Export Selection"))
-        button_close = wxButton(self.panel, wxID_CLOSE, _("Close"))
+        button_query = wx.Button(self.panel, ID_QUERY, _("Query"))
+        button_export = wx.Button(self.panel, ID_EXPORT, _("Export"))
+        button_exportSel = wx.Button(self.panel, ID_EXPORTSEL, _("Export Selection"))
+        button_close = wx.Button(self.panel, wx.ID_CLOSE, _("Close"))
 
         self.CreateStatusBar()
 
@@ -398,35 +407,35 @@ class QueryTableFrame(TableFrame):
 
         self.UpdateStatusText()
 
-        topBox = wxBoxSizer(wxVERTICAL)
+        topBox = wx.BoxSizer(wx.VERTICAL)
 
-        sizer = wxStaticBoxSizer(wxStaticBox(self.panel, -1, 
+        sizer = wx.StaticBoxSizer(wx.StaticBox(self.panel, -1,
                                   _("Selection")),
-                                  wxHORIZONTAL)
-        sizer.Add(self.combo_fields, 1, wxEXPAND|wxALL, 4)
-        sizer.Add(self.choice_comp, 0, wxALL, 4)
-        sizer.Add(self.combo_value, 1, wxEXPAND|wxALL, 4)
-        sizer.Add(self.choice_action, 0, wxALL, 4)
-        sizer.Add(button_query, 0, wxALL | wxALIGN_CENTER_VERTICAL, 4)
-        sizer.Add(40, 20, 0, wxALL, 4)
+                                  wx.HORIZONTAL)
+        sizer.Add(self.combo_fields, 1, wx.EXPAND|wx.ALL, 4)
+        sizer.Add(self.choice_comp, 0, wx.ALL, 4)
+        sizer.Add(self.combo_value, 1, wx.EXPAND|wx.ALL, 4)
+        sizer.Add(self.choice_action, 0, wx.ALL, 4)
+        sizer.Add(button_query, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 4)
+        sizer.Add( (40, 20), 0, wx.ALL, 4)
 
-        topBox.Add(sizer, 0, wxEXPAND|wxALL, 4)
-        topBox.Add(self.grid, 1, wxEXPAND|wxALL, 0)
+        topBox.Add(sizer, 0, wx.EXPAND|wx.ALL, 4)
+        topBox.Add(self.grid, 1, wx.EXPAND|wx.ALL, 0)
 
-        sizer = wxBoxSizer(wxHORIZONTAL)
-        sizer.Add(button_export, 0, wxALL, 4)
-        sizer.Add(button_exportSel, 0, wxALL, 4)
-        sizer.Add(60, 20, 1, wxALL|wxEXPAND, 4)
-        sizer.Add(button_close, 0, wxALL|wxALIGN_RIGHT, 4)
-        topBox.Add(sizer, 0, wxALL | wxEXPAND, 4)
+        sizer = wx.BoxSizer(wx.HORIZONTAL)
+        sizer.Add(button_export, 0, wx.ALL, 4)
+        sizer.Add(button_exportSel, 0, wx.ALL, 4)
+        sizer.Add( (60, 20), 1, wx.ALL|wx.EXPAND, 4)
+        sizer.Add(button_close, 0, wx.ALL|wx.ALIGN_RIGHT, 4)
+        topBox.Add(sizer, 0, wx.ALL | wx.EXPAND, 4)
 
         self.panel.SetAutoLayout(True)
         self.panel.SetSizer(topBox)
         topBox.Fit(self.panel)
         topBox.SetSizeHints(self.panel)
 
-        panelSizer = wxBoxSizer(wxVERTICAL)
-        panelSizer.Add(self.panel, 1, wxEXPAND, 0)
+        panelSizer = wx.BoxSizer(wx.VERTICAL)
+        panelSizer.Add(self.panel, 1, wx.EXPAND, 0)
         self.SetAutoLayout(True)
         self.SetSizer(panelSizer)
         panelSizer.Fit(self)
@@ -434,17 +443,17 @@ class QueryTableFrame(TableFrame):
 
         self.grid.SetFocus()
 
-        EVT_BUTTON(self, ID_QUERY, self.OnQuery)
-        EVT_BUTTON(self, ID_EXPORT, self.OnExport)
-        EVT_BUTTON(self, ID_EXPORTSEL, self.OnExportSel)
-        EVT_BUTTON(self, wxID_CLOSE, self.OnClose)
-        EVT_KEY_DOWN(self.grid, self.OnKeyDown)
+        self.Bind(wx.EVT_BUTTON, self.OnQuery, id=ID_QUERY)
+        self.Bind(wx.EVT_BUTTON, self.OnExport, id=ID_EXPORT)
+        self.Bind(wx.EVT_BUTTON, self.OnExportSel, id=ID_EXPORTSEL)
+        self.Bind(wx.EVT_BUTTON, self.OnClose, id=wx.ID_CLOSE)
+        self.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown, self.grid)
 
         self.grid.Subscribe(ROW_SELECTED, self.UpdateStatusText)
 
     def UpdateStatusText(self, rows=None):
-        self.SetStatusText(_("%i rows (%i selected), %i columns") 
-            % (self.grid.GetNumberRows(), 
+        self.SetStatusText(_("%i rows (%i selected), %i columns")
+            % (self.grid.GetNumberRows(),
                self.grid.GetNumberSelected(),
                self.grid.GetNumberCols()))
 
@@ -468,12 +477,12 @@ class QueryTableFrame(TableFrame):
                 value = self.table.Column(text)
 
             ids = self.table.SimpleQuery(
-                    self.table.Column(self.combo_fields.GetStringSelection()), 
-                    self.choice_comp.GetStringSelection(), 
+                    self.table.Column(self.combo_fields.GetStringSelection()),
+                    self.choice_comp.GetStringSelection(),
                     value)
 
             choice = self.choice_action.GetSelection()
-            
+
             #
             # what used to be nice code got became a bit ugly because
             # each time we select a row a message is sent to the grid
@@ -520,15 +529,15 @@ class QueryTableFrame(TableFrame):
 
         finally:
             ThubanEndBusyCursor()
-        
+
     def doExport(self, onlySelected):
-        
-        dlg = wxFileDialog(self, _("Export Table To"), ".", "",
+
+        dlg = wx.FileDialog(self, _("Export Table To"), ".", "",
                            _("DBF Files (*.dbf)|*.dbf|") +
                            _("CSV Files (*.csv)|*.csv|") +
                            _("All Files (*.*)|*.*"),
-                           wxSAVE|wxOVERWRITE_PROMPT)
-        if dlg.ShowModal() == wxID_OK:
+                           wx.SAVE|wx.OVERWRITE_PROMPT)
+        if dlg.ShowModal() == wx.ID_OK:
             filename = dlg.GetPath()
             type = os.path.basename(filename).split('.')[-1:][0]
             dlg.Destroy()
@@ -543,8 +552,8 @@ class QueryTableFrame(TableFrame):
             elif type.upper() == 'CSV':
                 table_to_csv(self.table, filename, records)
             else:
-                dlg = wxMessageDialog(None, "Unsupported format: %s" % type,
-                                      "Table Export", wxOK|wxICON_WARNING)
+                dlg = wx.MessageDialog(None, "Unsupported format: %s" % type,
+                                      "Table Export", wx.OK|wx.ICON_WARNING)
                 dlg.ShowModal()
                 dlg.Destroy()
         else:
