@@ -1,7 +1,8 @@
-# Copyright (c) 2001, 2002, 2003, 2004 by Intevation GmbH
+# Copyright (c) 2003-2005 by Intevation GmbH
 # Authors:
-# Bernhard Herzog <bh@intevation.de>
-# Frank Koormann <frank@intevation.de>
+# Bernhard Herzog <bh@intevation.de> (2003, 2004)
+# Jonathan Coles <jonathan@intevation.de> (2003)
+# Jan-Oliver Wagner <jan@intevation.de> (2004)
 #
 # This program is free software under the GPL (>=v2)
 # Read the file COPYING coming with Thuban for details.
@@ -10,14 +11,18 @@
 Classes for display of a map and interaction with it
 """
 
-__version__ = "$Revision: 1.16.2.1 $"
+__version__ = "$Revision: 2710 $"
+# $Source$
+# $Id: viewport.py 2710 2006-10-10 08:31:44Z dpinte $
 
+import sys
 from math import hypot
 
 from wxproj import point_in_polygon_shape, shape_centroid
 
 from Thuban.Model.messages import MAP_PROJECTION_CHANGED, \
-     LAYER_PROJECTION_CHANGED, TITLE_CHANGED
+     LAYER_PROJECTION_CHANGED, TITLE_CHANGED, \
+     MAP_LAYERS_ADDED, MAP_LAYERS_REMOVED
 from Thuban.Model.data import SHAPETYPE_POLYGON, SHAPETYPE_ARC, \
      SHAPETYPE_POINT, RAW_SHAPEFILE
 from Thuban.Model.label import ALIGN_CENTER, ALIGN_TOP, ALIGN_BOTTOM, \
@@ -28,7 +33,7 @@ from Thuban.Model.color import Transparent
 from selection import Selection
 
 from messages import LAYER_SELECTED, SHAPES_SELECTED, VIEW_POSITION, \
-                     SCALE_CHANGED, MAP_REPLACED
+     SCALE_CHANGED, MAP_REPLACED
 
 import hittest
 
@@ -223,6 +228,13 @@ class ViewPort(Conduit):
                          "HasSelectedShapes": "selection",
                          "SelectedShapes": "selection"}
 
+    # Some messages are forwarded from the currently shown map.  This is
+    # simply a list of the channels to forward.  The _subscribe_map and
+    # _unsubscribe_map methods use this to handle the forwarding.
+    forwarded_map_messages = (LAYER_PROJECTION_CHANGED, TITLE_CHANGED,
+                              MAP_PROJECTION_CHANGED,
+                              MAP_LAYERS_ADDED, MAP_LAYERS_REMOVED)
+
     def __init__(self, size = (400, 300)):
 
         self.size = size
@@ -308,14 +320,16 @@ class ViewPort(Conduit):
                           self.layer_projection_changed)
             map.Subscribe(MAP_PROJECTION_CHANGED,
                           self.map_projection_changed)
-            self.subscribe_forwarding(TITLE_CHANGED, map)
+            for channel in self.forwarded_map_messages:
+                self.subscribe_forwarding(channel, map)
 
     def _unsubscribe_map(self, map):
         """
         Internal: Unsubscribe from the messages subscribed to in _subscribe_map
         """
         if map is not None:
-            self.unsubscribe_forwarding(TITLE_CHANGED, map)
+            for channel in self.forwarded_map_messages:
+                self.unsubscribe_forwarding(channel, map)
             map.Unsubscribe(MAP_PROJECTION_CHANGED,
                             self.map_projection_changed)
             map.Unsubscribe(LAYER_PROJECTION_CHANGED,
@@ -397,8 +411,17 @@ class ViewPort(Conduit):
 
         # The window's center in projected coordinates assuming the new
         # scale/offset
-        pcenterx = (wwidth/2 - offset[0]) / scale
-        pcentery = (offset[1] - wheight/2) / scale
+        try:
+            pcenterx = (wwidth/2 - offset[0]) / scale
+            pcentery = (offset[1] - wheight/2) / scale
+        except ZeroDivisionError:
+            # scale was zero.  Probably a problem with the projections.
+            # printing an error message and return immediately. The user
+            # will hopefully be informed by the UI which displays a
+            # message for at least some of the projection problems.
+            print >>sys.stderr, "ViewPort.set_view_transform:", \
+                  "ZeroDivisionError, scale =", repr(scale)
+            return
 
         min_scale, max_scale = self.calc_min_max_scales(scale)
 
@@ -431,6 +454,17 @@ class ViewPort(Conduit):
         offx, offy = self.offset
         return ((x - offx) / self.scale, (offy - y) / self.scale)
 
+    def VisibleExtent(self):
+        """Return the extent of the visible region in projected coordinates
+
+        The return values is a tuple (llx, lly, urx, ury) describing the
+        region.
+        """
+        width, height = self.GetPortSizeTuple()
+        llx, lly = self.win_to_proj(0, height)
+        urx, ury = self.win_to_proj(width, 0)
+        return (llx, lly, urx, ury)
+
     def FitRectToWindow(self, rect):
         """Fit the rectangular region given by rect into the window.
 
@@ -466,7 +500,7 @@ class ViewPort(Conduit):
         Set the scale so that the layer fits exactly into the window and
         center it in the window.
         """
-        
+
         bbox = layer.LatLongBoundingBox()
         if bbox is not None:
             proj = self.map.GetProjection()
@@ -488,7 +522,7 @@ class ViewPort(Conduit):
 
             if bbox is not None:
                 if len(shapes) == 1 and layer.ShapeType() == SHAPETYPE_POINT:
-                    self.ZoomFactor(self.calc_min_max_scales()[1] / self.scale, 
+                    self.ZoomFactor(self.calc_min_max_scales()[1] / self.scale,
                                     self.proj_to_win(bbox[0], bbox[1]))
                 else:
                     self.FitRectToWindow(bbox)
@@ -723,14 +757,19 @@ class ViewPort(Conduit):
         # around the point. For layers containing points we have to
         # choose a larger size of the box we're testing against so
         # that we take the size of the markers into account
-        # FIXME: Once the markers are more flexible this part has to
-        # become more flexible too, of course
+        # The size of the box is determined by the largest symbol
+        # of the corresponding layer.
+        maxsize = 1
         if layer.ShapeType() == SHAPETYPE_POINT:
-            box = self.unprojected_rect_around_point(px, py, 5)
-        else:
-            box = self.unprojected_rect_around_point(px, py, 1)
+            for group in layer.GetClassification():
+                props = group.GetProperties()
+                if props.GetSize() > maxsize:
+                    maxsize = props.GetSize()
+        box = self.unprojected_rect_around_point(px, py, maxsize)
 
+        # determine the function that does the hit test based on the layer
         hittester = self._get_hit_tester(layer)
+
         for shape in layer.ShapesInRegion(box):
             if field:
                 record = table.ReadRowAsDict(shape.ShapeID())
@@ -738,7 +777,13 @@ class ViewPort(Conduit):
                 props = group.GetProperties()
                 filled = props.GetFill() is not Transparent
                 stroked = props.GetLineColor() is not Transparent
-            hit = hittester(layer, shape, filled, stroked, px, py)
+
+            if layer.ShapeType() == SHAPETYPE_POINT:
+                hit = hittester(layer, shape, filled, stroked,
+                                px, py, size = props.GetSize())
+            else:
+                hit = hittester(layer, shape, filled, stroked, px, py)
+
             if hit:
                 return shape.ShapeID()
         return None
@@ -802,14 +847,18 @@ class ViewPort(Conduit):
                                    -y * scale + offy))
         return result
 
-    def _hit_point(self, layer, shape, filled, stroked, px, py):
+    def _hit_point(self, layer, shape, filled, stroked, px, py, size = 5):
         """Internal: return whether a click at (px,py) hits the point shape
 
         The filled and stroked parameters determine whether the shape is
         assumed to be filled or stroked respectively.
+
+        size -- defines the size of the point symbol. For the hitting
+                test it is assumed the symbol is a circle of this size
+                (radius).
         """
         x, y = self.projected_points(layer, shape.Points())[0][0]
-        return hypot(px - x, py - y) < 5 and (filled or stroked)
+        return hypot(px - x, py - y) < size and (filled or stroked)
 
     def _hit_arc(self, layer, shape, filled, stroked, px, py):
         """Internal: return whether a click at (px,py) hits the arc shape
@@ -914,43 +963,8 @@ class ViewPort(Conduit):
             label_layer.RemoveLabel(shape_index)
             return True
         elif layer is not None and text:
-            proj = self.map.projection
-            if proj is not None:
-                map_proj = proj
-            else:
-                map_proj = None
-            proj = layer.projection
-            if proj is not None:
-                layer_proj = proj
-            else:
-                layer_proj = None
-
-            shapetype = layer.ShapeType()
-            if shapetype == SHAPETYPE_POLYGON:
-                shapefile = layer.ShapeStore().Shapefile().cobject()
-                x, y = shape_centroid(shapefile, shape_index,
-                                      map_proj, layer_proj, 1, 1, 0, 0)
-                if map_proj is not None:
-                    x, y = map_proj.Inverse(x, y)
-            else:
-                shape = layer.Shape(shape_index)
-                if shapetype == SHAPETYPE_POINT:
-                    x, y = shape.Points()[0][0]
-                else:
-                    # assume SHAPETYPE_ARC
-                    points = shape.Points()[0]
-                    x, y = points[len(points) / 2]
-                if layer_proj is not None:
-                    x, y = layer_proj.Inverse(x, y)
-            if shapetype == SHAPETYPE_POINT:
-                halign = ALIGN_LEFT
-                valign = ALIGN_CENTER
-            elif shapetype == SHAPETYPE_POLYGON:
-                halign = ALIGN_CENTER
-                valign = ALIGN_CENTER
-            elif shapetype == SHAPETYPE_ARC:
-                halign = ALIGN_LEFT
-                valign = ALIGN_CENTER
+            (x, y, halign, valign)  = layer.GetLabelPosFromShape(self.map, \
+                                                             shape_index)
             label_layer.AddLabel(x, y, text,
                                  halign = halign, valign = valign)
             return True
@@ -976,12 +990,12 @@ def output_transform(canvas_scale, canvas_offset, canvas_size, device_extend):
         width = width * 0.9
         map_height = width - 2*distance
         map_width = map_height
-    
-    mapregion = (distance, distance, 
+
+    mapregion = (distance, distance,
                  distance+map_width, distance+map_height)
 
     canvas_width, canvas_height = canvas_size
-    
+
     scalex = map_width / (canvas_width/canvas_scale)
     scaley = map_height / (canvas_height/canvas_scale)
     scale = min(scalex, scaley)
