@@ -1,4 +1,4 @@
-# Copyright (C) 2003, 2004 by Intevation GmbH
+# Copyright (C) 2003, 2004, 2005, 2006 by Intevation GmbH
 # Authors:
 # Bernhard Herzog <bh@intevation.de>
 #
@@ -7,9 +7,9 @@
 
 """Support module for tests that use a live PostGIS database"""
 
-__version__ = "$Revision: 1.7.2.3 $"
-# $Source: /thubanrepository/thuban/test/postgissupport.py,v $
-# $Id: postgissupport.py,v 1.7.2.3 2004/12/16 20:34:03 bh Exp $
+__version__ = "$Revision: 2717 $"
+# $Source$
+# $Id: postgissupport.py 2717 2007-01-01 21:44:05Z bernhard $
 
 import sys
 import os
@@ -85,6 +85,13 @@ def run_boolean_command(command):
 #       PostgreSQL and database
 #
 
+_pg_ctl_command = "pg_ctl"
+_initdb_command = "initdb"
+# Example Values for Debian (upcomind Etch) postgresql-8.1 
+# TODO: detect which values to use here.
+#_pg_ctl_command = "/usr/lib/postgresql/8.1/bin/pg_ctl"
+#_initdb_command = "/usr/lib/postgresql/8.1/bin/initdb"
+
 class PostgreSQLServer:
 
     """A PostgreSQL server
@@ -142,13 +149,13 @@ class PostgreSQLServer:
             shutil.rmtree(self.dbdir)
         os.mkdir(self.dbdir)
 
-        run_command(["initdb", "-D", self.dbdir, "-U", self.admin_name],
+        run_command([_initdb_command, "-D", self.dbdir, "-U", self.admin_name],
                     os.path.join(self.dbdir, "initdb.log"))
 
         extra_opts = "-p %d" % self.port
         if self.socket_dir is not None:
             extra_opts += " -k %s" % self.socket_dir
-        run_command(["pg_ctl", "-D", self.dbdir,
+        run_command([_pg_ctl_command, "-D", self.dbdir,
                      "-l", os.path.join(self.dbdir, "logfile"),
                      "-o", extra_opts, "start"],
                     os.path.join(self.dbdir, "pg_ctl-start.log"))
@@ -196,23 +203,25 @@ class PostgreSQLServer:
         instance, if the server has been started manually for debugging
         purposes after a test suite run.
         """
-        return run_boolean_command(["pg_ctl", "-D", self.dbdir, "status"])
+        return run_boolean_command([_pg_ctl_command, "-D", self.dbdir, "status"])
 
     def shutdown(self):
         """Stop the postmaster running for self.dbdir"""
-        run_command(["pg_ctl", "-m", "fast", "-D", self.dbdir, "stop"],
+        run_command([_pg_ctl_command, "-m", "fast", "-D", self.dbdir, "stop"],
                     os.path.join(self.dbdir, "pg_ctl-stop.log"))
 
-    def new_postgis_db(self, dbname, tables = None, reference_systems = None):
+    def new_postgis_db(self, dbname, tables = None, reference_systems = None,
+                       views = None):
         """Create and return a new PostGISDatabase object using self as server
         """
         db = PostGISDatabase(self, self.postgis_sql, dbname, tables = tables,
-                             reference_systems = reference_systems)
+                             reference_systems = reference_systems,
+                             views = views)
         db.initdb()
         self.known_dbs[dbname] = db
         return db
 
-    def get_static_data_db(self, dbname, tables, reference_systems):
+    def get_static_data_db(self, dbname, tables, reference_systems, views):
         """Return a PostGISDatabase for a database with the given static data
 
         If no databasse of the name dbname exists, create a new one via
@@ -230,11 +239,11 @@ class PostgreSQLServer:
         """
         db = self.known_dbs.get(dbname)
         if db is not None:
-            if db.has_data(tables, reference_systems):
+            if db.has_data(tables, reference_systems, views):
                 return db
             raise ValueError("PostGISDatabase named %r doesn't have tables %r"
                              % (dbname, tables))
-        return self.new_postgis_db(dbname, tables, reference_systems)
+        return self.new_postgis_db(dbname, tables, reference_systems, views)
 
     def get_default_static_data_db(self):
         dbname = "PostGISStaticTests"
@@ -242,7 +251,8 @@ class PostgreSQLServer:
         tables = [
             # Direct copies of the shapefiles. The shapeids are exactly
             # the same, except where changed with "gid_offset", of
-            # course
+            # course.  Note that the test implementation requires that
+            # all the landmard tables use an gid_offset of 1000.
             ("landmarks", os.path.join("..", "Data", "iceland",
                                        "cultural_landmark-point.shp"),
              [("gid_offset", 1000)]),
@@ -267,8 +277,17 @@ class PostgreSQLServer:
                                        "cultural_landmark-point.shp"),
              [("gid_offset", 1000),
               ("srid", 1)]),
+
+            # Copy of landmarks with a gid column called "point_id" instead
+            # of "gid" and using an srid != -1.
+            ("landmarks_point_id", os.path.join("..", "Data", "iceland",
+                                                "cultural_landmark-point.shp"),
+             [("gid_offset", 1000),
+              ("srid", 1),
+              ("gid_column", "point_id")]),
             ]
-        return self.get_static_data_db(dbname, tables, srids)
+        views = [("v_landmarks", "SELECT * FROM landmarks_point_id")]
+        return self.get_static_data_db(dbname, tables, srids, views)
 
     def connection_params(self, user):
         """Return the connection parameters for the given user
@@ -356,7 +375,7 @@ class PostgreSQLServer:
         f = open(os.path.join(self.dbdir, "pg_hba.conf"), "w")
         f.write(contents)
         f.close()
-        run_command(["pg_ctl", "-D", self.dbdir, "reload"],
+        run_command([_pg_ctl_command, "-D", self.dbdir, "reload"],
                     os.path.join(self.dbdir, "pg_ctl-reload.log"))
 
 
@@ -376,7 +395,7 @@ class PostGISDatabase:
     """A PostGIS database in a PostgreSQLServer"""
 
     def __init__(self, server, postgis_sql, dbname, tables = None,
-                 reference_systems = ()):
+                 reference_systems = (), views = None):
         """Initialize the PostGISDatabase
 
         Parameters:
@@ -384,8 +403,8 @@ class PostGISDatabase:
             server -- The PostgreSQLServer instance containing the
                 database
 
-            postgis_sql -- Filename of the postgis.sql file with the
-                postgis initialization code
+            postgis_sql -- Filename of the sql file with the postgis
+                initialization code
 
             dbname -- The name of the database
 
@@ -402,11 +421,19 @@ class PostGISDatabase:
                 (srid, params) pairs where srid is the srid defined by
                 the proj4 paramter string params.  The srid can be given
                 as an extra parameter in the tables list.
+
+            views -- Optional description of views.  If given it should
+                be a list of (viewname, select_stmt) pairs where
+                viewname is the name of the view to be created and
+                select_stmt is the select statement to use as the basis.
+                The views will be created after the tables and may refer
+                to them in the select_stmt.
         """
         self.server = server
         self.postgis_sql = postgis_sql
         self.dbname = dbname
         self.tables = tables
+        self.views = views
         if reference_systems:
             self.reference_systems = reference_systems
         else:
@@ -462,9 +489,19 @@ class PostGISDatabase:
                 tablename, shapefile, kw = unpack(info)
                 upload_shapefile(shapefile, self, tablename, **kw)
 
-    def has_data(self, tables, reference_systems):
+        if self.views is not None:
+            for viewname, select_stmt in self.views:
+                self.server.execute_sql(self.dbname, "admin",
+                                        "CREATE VIEW %s AS %s" % (viewname,
+                                                                  select_stmt))
+                self.server.execute_sql(self.dbname, "admin",
+                                        "GRANT SELECT ON %s TO PUBLIC;"
+                                        % viewname)
+
+    def has_data(self, tables, reference_systems, views):
         return (self.tables == tables
-                and self.reference_systems == reference_systems)
+                and self.reference_systems == reference_systems
+                and self.views == views)
 
 
 def find_postgis_sql():
@@ -478,19 +515,33 @@ def find_postgis_sql():
     $bindir/../share/postgresql/.
 
     Furthermore, different versions of postgis place the file in
-    slightly different locations.  For instance:
+    slightly different locations or may even use different names.  For
+    instance:
 
       postgis 0.7.5        $datadir/contrib/postgis.sql
       postgis 0.8.1        $datadir/postgis.sql
+      postgis 1.0.0-rc1    $datadir/lwpostgis.sql
+      postgis 1.0.0-rc4    $datadir/contrib/lwpostgis.sql
 
     To support both versions, we look in both places and return the
-    first one found (looking under contrib first).  If the file is not
-    found the return value is None.
+    first one found (looking under contrib first).  
+
+    Debian (umcoming Etch) can do several version of postgresql
+    and thus has changed the paths. We try one location
+    in datadir2 only for Debian Etch postgresql-8.1.
+
+    If the file is not found the return value is None.
     """
     bindir = run_config_script("pg_config --bindir").strip()
     datadir = os.path.join(bindir, "..", "share", "postgresql")
+    datadir2 = os.path.join("/", "usr", "share", "postgresql-8.1-postgis")
+
     for filename in [os.path.join(datadir, "contrib", "postgis.sql"),
-                     os.path.join(datadir, "postgis.sql")]:
+                     os.path.join(datadir, "postgis.sql"),
+                     os.path.join(datadir, "lwpostgis.sql"),
+                     os.path.join(datadir, "contrib", "lwpostgis.sql"),
+                     os.path.join(datadir2, "lwpostgis.sql") \
+                    ]:
         if os.path.exists(filename):
             return filename
 
@@ -543,7 +594,7 @@ def reason_for_not_running_tests():
         return "Can't run PostGIS test because popen2.Popen4 does not exist"
 
     try:
-        run_command(["pg_ctl", "--help"], None)
+        run_command([_pg_ctl_command, "--help"], None)
     except RuntimeError:
         return "Can't run PostGIS tests because pg_ctl fails"
 
@@ -634,7 +685,7 @@ wkt_converter = {
     }
 
 def upload_shapefile(filename, db, tablename, force_wkt_type = None,
-                     gid_offset = 0, srid = -1):
+                     gid_offset = 0, gid_column = "gid", srid = -1):
     """Upload a shapefile into a new database table
 
     Parameters:
@@ -653,8 +704,17 @@ def upload_shapefile(filename, db, tablename, force_wkt_type = None,
     gid_offset -- A number to add to the shapeid to get the value for
                 the gid column (default 0)
 
+    gid_column -- The name of the column with the shape ids.  Default
+                  'gid'.  If None, no gid column will be created.  The
+                  name is directly used in SQL statements, so if it
+                  contains unusualy characters the caller should provide
+                  a suitable quoted string.
+
     srid -- The srid of the spatial references system used by the table
             and the data
+
+    The tables will be explicitely created WITH OIDS. This has been
+    default for PostgreSQL <8.0 and some tests relied on it (end of 2006).
     """
     import dbflib, shapelib
 
@@ -679,14 +739,22 @@ def upload_shapefile(filename, db, tablename, force_wkt_type = None,
                dbflib.FTInteger: "INTEGER",
                dbflib.FTDouble: "DOUBLE PRECISION"}
 
-    insert_formats = ["%(gid)s"]
-    fields = ["gid INT"]
+    insert_formats = []
+    if gid_column:
+        insert_formats.append("%(gid)s")
+
+    fields = []
+    fields_decl = []
+    if gid_column:
+        fields.append(gid_column)
+        fields_decl.append("%s INT" % gid_column)
     for i in range(dbf.field_count()):
         ftype, name, width, prec = dbf.field_info(i)
-        fields.append("%s %s" % (name, typemap[ftype]))
+        fields.append(name)
+        fields_decl.append("%s %s" % (name, typemap[ftype]))
         insert_formats.append("%%(%s)s" % name)
-    stmt = "CREATE TABLE %s (\n    %s\n);" % (tablename,
-                                              ",\n    ".join(fields))
+    stmt = "CREATE TABLE %s (\n    %s\n) WITH OIDS ;" % (tablename,
+                                              ",\n    ".join(fields_decl))
     cursor.execute(stmt)
     #print stmt
 
@@ -699,16 +767,17 @@ def upload_shapefile(filename, db, tablename, force_wkt_type = None,
     cursor.execute("select AddGeometryColumn('%(dbname)s',"
                    "'%(tablename)s', 'the_geom', %(srid)d, '%(wkttype)s', 2);"
                    % locals())
-
+    fields.append("the_geom")
     insert_formats.append("GeometryFromText(%(the_geom)s, %(srid)d)")
 
-    insert = ("INSERT INTO %s VALUES (%s)"
-              % (tablename, ", ".join(insert_formats)))
+    insert = ("INSERT INTO %s (%s) VALUES (%s)"
+              % (tablename, ", ".join(fields), ", ".join(insert_formats)))
 
     for i in range(numshapes):
         data = dbf.read_record(i)
         data["tablename"] = tablename
-        data["gid"] = i + gid_offset
+        if gid_column:
+            data["gid"] = i + gid_offset
         data["srid"] = srid
         data["the_geom"] = convert(shp.read_object(i).vertices())
         #print insert % data
