@@ -1,4 +1,4 @@
-# Copyright (C) 2003, 2004 by Intevation GmbH
+# Copyright (C) 2003, 2004, 2005, 2006 by Intevation GmbH
 # Authors:
 # Bernhard Herzog <bh@intevation.de>
 #
@@ -98,22 +98,41 @@ class TestPostGISConnection(unittest.TestCase):
         # An empty database doesn't have any GIS tables
         self.assertEquals(db.GeometryTables(), [])
 
-    def test_gis_tables_non_empty(self):
-        """Test PostGISConnection.GISTables() on non-empty DB"""
+    def test_gis_tables_with_views_and_tables(self):
         db = PostGISConnection(dbname = self.dbname,
                                **self.server.connection_params("user"))
 
         conn = psycopg.connect("dbname=%s " % self.dbname
                                + self.server.connection_string("admin"))
         cursor = conn.cursor()
-        cursor.execute("CREATE TABLE test (A INT);")
-        cursor.execute("SELECT AddGeometryColumn(%(dbname)s, 'test',"
-                       " 'geometry', -1, 'POINT', 2);",
+        # First a normal table, i.e. one without a geometry column
+        # we need an explicit WITH OIDS for postgresql >= 8.0
+        cursor.execute("CREATE TABLE normal (A INT, B VARCHAR) WITH OIDS;"
+                       "GRANT SELECT ON normal TO PUBLIC;")
+
+        # Then a table with a geometry column
+        # we need an explicit WITH OIDS for postgresql >= 8.0
+        cursor.execute("CREATE TABLE geo (A INT) WITH OIDS;"
+                       "GRANT SELECT ON geo TO PUBLIC;")
+        cursor.execute("SELECT AddGeometryColumn(%(dbname)s, 'geo',"
+                       " 'the_geom', -1, 'POINT', 2);",
                        {"dbname": self.dbname})
+
+        # And create a view containing a geometry column
+        cursor.execute("CREATE VIEW my_view AS"
+                       " (SELECT * FROM geo WHERE a > 100);"
+                       "GRANT SELECT ON my_view TO PUBLIC;")
         conn.commit()
 
-        # An empty database doesn't have any GIS tables
-        self.assertEquals(db.GeometryTables(), ["test"])
+        self.assertEquals(db.GeometryTables(), ["geo", "my_view"])
+        self.assertEquals(db.table_columns("geo"),
+                          [("oid", FIELDTYPE_INT), ("a", FIELDTYPE_INT),
+                           ("the_geom", "geometry")])
+        self.assertEquals(db.table_columns("my_view"),
+                          [("a", FIELDTYPE_INT), ("the_geom", "geometry")])
+        self.assertEquals(db.table_columns("normal"),
+                          [("oid", FIELDTYPE_INT), ("a", FIELDTYPE_INT),
+                           ("b", FIELDTYPE_STRING)])
 
 
 class TestPostgisDBExceptions(unittest.TestCase):
@@ -331,6 +350,45 @@ class TestPostGISSpecialCases(unittest.TestCase):
                           [1])
 
 
+    def test_simple_error_handling(self):
+        """Test some simple error handling.
+
+        If an error happens in, say the SimpleQuery method, a subsequent
+        ReadValue call should still succeed.
+        """
+        conn = psycopg.connect("dbname=%s " % self.dbname
+                               + self.server.connection_string("admin"))
+        cursor = conn.cursor()
+        cursor.execute("CREATE TABLE some_table"
+                       " (gid INTEGER PRIMARY KEY, length float);"
+                       "INSERT INTO some_table VALUES (1, 3.5);")
+        cursor.execute("GRANT SELECT ON some_table TO PUBLIC;")
+        conn.commit()
+        conn.close()
+
+        db = PostGISConnection(dbname = self.dbname,
+                               **self.server.connection_params("user"))
+        table = PostGISTable(db, "some_table", id_column = "gid")
+
+        # trying to compare the length to a string should fail with an
+        # exception.
+        self.assertRaises(psycopg.ProgrammingError,
+                          table.SimpleQuery,
+                          table.Column("length"), "<=", "abc")
+
+        # After the exception, doing a query on the same table should
+        # work. Since it's the same database connection as before, it
+        # can fail if no rollback has been done or the connection isn't
+        # in autocommit mode.  Typical error message is
+        #
+        #     ERROR: current transaction is aborted, commands ignored
+        #     until end of transaction block
+        try:
+            self.assertEquals(table.ReadValue(1, "length"), 3.5)
+        except psycopg.ProgrammingError, val:
+            self.fail("table didn't handle exception properly (%s)" % val)
+
+
 class PostGISStaticTests(unittest.TestCase, support.FloatComparisonMixin):
 
     """Base class for PostGIS tests with static data."""
@@ -357,12 +415,29 @@ class PostGISStaticTests(unittest.TestCase, support.FloatComparisonMixin):
         self.db.Close()
 
 
-class TestPostGISTable(PostGISStaticTests):
+class TableTests:
 
-    def setUp(self):
-        """Extend inherited method to set self.table to a PostGISTable"""
-        PostGISStaticTests.setUp(self)
-        self.table = PostGISTable(self.db, "landmarks")
+    """Mix-in class for the PostGISTable tests
+
+    The tests defined here make some assumptions about the table and the
+    data in it:
+
+        self.table should be the PostGISTable instance to test.  Derived
+        classes should set this up in setUp.
+
+        self.db should be the DB connection object used by self.table.
+
+        self.table_name should be the name of the table in the database.
+        The tests assume that e.g. the title of self.table can be
+        derived from this.
+
+        self.gid_column should be the name of the gid column used when
+        self.table was instantiated.
+
+        The data in the table should be the data of the
+        cultural_landmark-point shapefile with the shape ids used to get
+        the contents of the gid column by adding 1000.
+    """
 
     def test_dbconn(self):
         """Test PostGISTable.DBConnection()"""
@@ -370,12 +445,12 @@ class TestPostGISTable(PostGISStaticTests):
 
     def test_dbname(self):
         """Test PostGISTable.TableName()"""
-        self.assertEquals(self.table.TableName(), "landmarks")
+        self.assertEquals(self.table.TableName(), self.table_name)
 
     def test_title(self):
         """test PostGISTable.Title()"""
         # The title is currently equal to the tablename
-        self.assertEquals(self.table.Title(), "landmarks")
+        self.assertEquals(self.table.Title(), self.table_name)
 
     def test_dependencies(self):
         """Test PostGISTable.Dependencies()"""
@@ -395,7 +470,7 @@ class TestPostGISTable(PostGISStaticTests):
     def test_columns(self):
         """Test PostGISTable.Columns()"""
         self.assertEquals(len(self.table.Columns()), 7)
-        self.assertEquals(self.table.Columns()[0].name, "gid")
+        self.assertEquals(self.table.Columns()[0].name, self.gid_column)
         self.assertEquals(self.table.Columns()[0].type, FIELDTYPE_INT)
         self.assertEquals(self.table.Columns()[0].index, 0)
         self.assertEquals(self.table.Columns()[1].name, "area")
@@ -419,7 +494,7 @@ class TestPostGISTable(PostGISStaticTests):
     def test_read_row_as_dict(self):
         """Test PostGISTable.ReadRowAsDict()"""
         self.assertEquals(self.table.ReadRowAsDict(1003),
-                          {"gid": 1003,
+                          {self.gid_column: 1003,
                            "area": 0.0,
                            "perimeter": 0.0,
                            "clpoint_": 4,
@@ -430,7 +505,7 @@ class TestPostGISTable(PostGISStaticTests):
     def test_read_row_as_dict_row_count_mode(self):
         """Test PostGISTable.ReadRowAsDict() row count address mode"""
         self.assertEquals(self.table.ReadRowAsDict(3, row_is_ordinal = 1),
-                          {"gid": 1003,
+                          {self.gid_column: 1003,
                            "area": 0.0,
                            "perimeter": 0.0,
                            "clpoint_": 4,
@@ -483,9 +558,41 @@ class TestPostGISTable(PostGISStaticTests):
                           [1028, 1029, 1030, 1031, 1032, 1033])
 
 
+class TestPostGISTable(TableTests, PostGISStaticTests):
+
+    def setUp(self):
+        """Extend inherited method to set self.table to a PostGISTable"""
+        PostGISStaticTests.setUp(self)
+        self.table_name = "landmarks"
+        self.gid_column = "gid"
+
+        # Note that omit the gid column here to the backwards compatible
+        # interface of the constructor where the gid_column defaults to
+        # "gid"
+        self.table = PostGISTable(self.db, self.table_name)
+
+
+class TestPostGISTableExplicitGIDColumn(TableTests, PostGISStaticTests):
+
+    def setUp(self):
+        """Extend inherited method to set self.table to a PostGISTable"""
+        PostGISStaticTests.setUp(self)
+        self.table_name = "landmarks_point_id"
+        self.gid_column = "point_id"
+        self.table = PostGISTable(self.db, self.table_name,
+                                  id_column = self.gid_column)
+
+
 class PointTests:
 
-    """Mix-in class for the tests on POINT tables"""
+    """Mix-in class for the tests on POINT tables
+
+    The methods in this class expect self.store to be the shapestore
+    used for testing.  Derived classes should initialize self.store with
+    PostGISShapeStore in setUp.  Another assumption is that the data in
+    self.store is effectively the cultural_landmark-point shapefile with
+    shape ids increased by 1000.
+    """
 
     #
     # First, some tests that should be independend of the shapetype, so
@@ -516,6 +623,13 @@ class PointTests:
         self.assertEquals([s.ShapeID() for s in self.store.AllShapes()],
                           range(1000, 1000 + self.store.NumShapes()))
 
+    def test_id_column(self):
+        self.assertEquals(self.store.IDColumn().name, self.id_column)
+
+    def test_geometry_column(self):
+        self.assertEquals(self.store.GeometryColumn().name,
+                          self.geometry_column)
+
     #
     # Shapetype specific tests
     #
@@ -562,6 +676,8 @@ class TestPostGISShapestorePoint(PointTests, PostGISStaticTests):
     def setUp(self):
         """Extend inherited method to set self.table to a PostGISShapeStore"""
         PostGISStaticTests.setUp(self)
+        self.id_column = "gid"
+        self.geometry_column = "the_geom"
         self.store = PostGISShapeStore(self.db, "landmarks")
 
 
@@ -573,9 +689,91 @@ class TestPostGISShapestorePointSRID(PointTests, PostGISStaticTests):
     def setUp(self):
         """Extend inherited method to set self.table to a PostGISShapeStore"""
         PostGISStaticTests.setUp(self)
+        self.id_column = "gid"
+        self.geometry_column = "the_geom"
         self.store = PostGISShapeStore(self.db, "landmarks_srid")
 
 
+class TestPostGISShapestorePointExplicitGIDColumn(PointTests,
+                                                  PostGISStaticTests):
+
+    """Tests for PostGISShapeStores with POINT data and explicit gid column"""
+
+    def setUp(self):
+        """Extend inherited method to set self.table to a PostGISShapeStore"""
+        PostGISStaticTests.setUp(self)
+        self.id_column = "point_id"
+        self.geometry_column = "the_geom"
+        self.store = PostGISShapeStore(self.db, "landmarks_point_id",
+                                       id_column = "point_id")
+
+
+class TestPostGISShapestorePointOIDAsGIDColumn(PointTests, PostGISStaticTests):
+
+    """Tests for PostGISShapeStores with POINT data using OID as gid column"""
+
+    def setUp(self):
+        """Extend inherited method to set self.table to a PostGISShapeStore"""
+        PostGISStaticTests.setUp(self)
+        self.id_column = "oid"
+        self.geometry_column = "the_geom"
+        self.store = PostGISShapeStore(self.db, "landmarks_point_id",
+                                       id_column = self.id_column,
+                                       geometry_column = self.geometry_column)
+
+    def find_test_shape_id(self):
+        """Return the id of an interesting shape for some test cases.
+
+        This test uses OIDs so we can't easily hard wire ids in the test
+        cases
+        """
+        # get the id of an interesting shape
+        shapes = self.store.ShapesInRegion((-20.0, 64.0, -24.0, 67))
+        return list(shapes)[0].ShapeID()
+
+
+    # Override a few tests that won't work with this table because they
+    # require knowledge of specific IDs
+
+    def test_shape_shapeid(self):
+        """Test PostGISShapeStore.Shape(i).ShapeID() with POINT shapes"""
+        gid = self.find_test_shape_id()
+        self.assertEquals(self.store.Shape(gid).ShapeID(), gid)
+
+    def test_shape_points(self):
+        """Test PostGISShapeStore.Shape(i).Points() with POINT shapes"""
+        gid = self.find_test_shape_id()
+        self.assertPointListEquals(self.store.Shape(gid).Points(),
+                                   [[(-22.711074829101562, 66.36572265625)]])
+
+    def test_shape_raw_data(self):
+        """Test PostGISShapeStore.Shape(i).RawData() with POINT shapes"""
+        gid = self.find_test_shape_id()
+        self.assertEquals(self.store.Shape(gid).RawData(),
+                          'POINT(-22.7110748291016 66.36572265625)')
+
+    def test_shapes_in_region(self):
+        """Test PostGISShapeStore:ShapesInRegion() with POINT shapes"""
+        shapes = self.store.ShapesInRegion((-20.0, 64.0, -24.0, 67))
+        self.assertEquals(len(list(shapes)), 7)
+
+    def test_all_shapes(self):
+        """Test PostGISShapeStore.AllShapes()"""
+        self.assertEquals(len(list(self.store.AllShapes())), 34)
+
+
+class TestPostGISShapestorePointFromViews(PointTests, PostGISStaticTests):
+
+    """Tests for PostGISShapeStores with POINT data in a view"""
+
+    def setUp(self):
+        """Extend inherited method to set self.table to a PostGISShapeStore"""
+        PostGISStaticTests.setUp(self)
+        self.id_column = "point_id"
+        self.geometry_column = "the_geom"
+        self.store = PostGISShapeStore(self.db, "v_landmarks",
+                                       id_column = "point_id")
+
 
 class LineStringTests:
 
