@@ -1,4 +1,4 @@
-# Copyright (c) 2002, 2003, 2004 by Intevation GmbH
+# Copyright (c) 2002, 2003, 2004, 2005 by Intevation GmbH
 # Authors:
 # Bernhard Herzog <bh@intevation.de>
 #
@@ -9,9 +9,9 @@
 Test saving a thuban session as XML
 """
 
-__version__ = "$Revision: 1.32 $"
-# $Source: /thubanrepository/thuban/test/test_save.py,v $
-# $Id: test_save.py,v 1.32 2004/02/11 09:05:40 bh Exp $
+__version__ = "$Revision: 2688 $"
+# $Source$
+# $Id: test_save.py 2688 2006-06-30 12:27:20Z frank $
 
 import os
 import unittest
@@ -25,6 +25,7 @@ support.initthuban()
 
 import dbflib
 
+from Thuban import internal_from_unicode
 from Thuban.Lib.fileutil import relative_filename
 from Thuban.Model.save import XMLWriter, save_session, sort_data_stores
 from Thuban.Model.session import Session
@@ -33,10 +34,10 @@ from Thuban.Model.layer import Layer, RasterLayer
 from Thuban.Model.proj import Projection
 from Thuban.Model.table import DBFTable
 from Thuban.Model.transientdb import TransientJoinedTable
-from Thuban.Model.data import DerivedShapeStore
+from Thuban.Model.data import DerivedShapeStore, SHAPETYPE_ARC
 
 from Thuban.Model.classification import ClassGroupSingleton, ClassGroupRange, \
-    ClassGroupProperties
+    ClassGroupPattern, ClassGroupProperties
 
 from Thuban.Model.range import Range
 
@@ -53,9 +54,9 @@ class XMLWriterTest(unittest.TestCase):
         eq(writer.encode("hello world"), "hello world")
         eq(writer.encode(unicode("hello world")), unicode("hello world"))
 
-        eq(writer.encode("\x80\x90\xc2\x100"), 
+        eq(writer.encode(internal_from_unicode(u"\x80\x90\xc2\x100")),
                          "\xc2\x80\xc2\x90\xc3\x82\x100")
-        eq(writer.encode(u"\x80\x90\xc2\x100"), 
+        eq(writer.encode(u"\x80\x90\xc2\x100"),
                          "\xc2\x80\xc2\x90\xc3\x82\x100")
         eq(writer.encode(u"\xFF5E"), "\xc3\xbf5E")
 
@@ -65,7 +66,7 @@ class XMLWriterTest(unittest.TestCase):
 class SaveSessionTest(unittest.TestCase, support.FileTestMixin,
                       xmlsupport.ValidationTest):
 
-    dtd = "http://thuban.intevation.org/dtds/thuban-1.0.0.dtd"
+    dtd = "http://thuban.intevation.org/dtds/thuban-1.1-dev.dtd"
     thubanids = [((dtd, n), (None, "id")) for n in
                  ["fileshapesource", "filetable", "jointable",
                   "derivedshapesource", "dbshapesource", "dbconnection"]]
@@ -111,9 +112,9 @@ class SaveSessionTest(unittest.TestCase, support.FileTestMixin,
         file.close()
         self.compare_xml(written_contents,
                          '<?xml version="1.0" encoding="UTF-8"?>\n'
-                         '<!DOCTYPE session SYSTEM "thuban-1.0.dtd">\n'
+                         '<!DOCTYPE session SYSTEM "thuban-1.1.dtd">\n'
                          '<session title="empty session" '
-         'xmlns="http://thuban.intevation.org/dtds/thuban-1.0.0.dtd">'
+         'xmlns="http://thuban.intevation.org/dtds/thuban-1.1-dev.dtd">'
                          '\n</session>\n')
 
         self.validate_data(written_contents)
@@ -141,9 +142,9 @@ class SaveSessionTest(unittest.TestCase, support.FileTestMixin,
         written_contents = file.read()
         file.close()
         expected_template = '''<?xml version="1.0" encoding="UTF-8"?>
-        <!DOCTYPE session SYSTEM "thuban-1.0.dtd">
+        <!DOCTYPE session SYSTEM "thuban-1.1.dtd">
         <session title="single map&amp;layer"
-           xmlns="http://thuban.intevation.org/dtds/thuban-1.0.0.dtd">
+           xmlns="http://thuban.intevation.org/dtds/thuban-1.1-dev.dtd">
             <fileshapesource id="D1"
                 filename="../../Data/iceland/political.shp"
                 filetype="shapefile"/>
@@ -155,8 +156,14 @@ class SaveSessionTest(unittest.TestCase, support.FileTestMixin,
                     <parameter value="datum=WGS84"/>
                     <parameter value="units=m"/>
                 </projection>
-                <layer title="My Layer" shapestore="D1"
-                fill="None" stroke="#000000" stroke_width="1" visible="%s"/>
+                <layer title="My Layer" shapestore="D1" visible="%s">
+                    <classification>
+                        <clnull label="">
+                            <cldata fill="None" stroke="#000000"
+		    	        stroke_width="1"/>
+                        </clnull>
+                    </classification>
+                </layer>
             </map>
         </session>'''
 
@@ -203,9 +210,9 @@ class SaveSessionTest(unittest.TestCase, support.FileTestMixin,
         written_contents = file.read()
         file.close()
         expected_contents = '''<?xml version="1.0" encoding="UTF-8"?>
-        <!DOCTYPE session SYSTEM "thuban-1.0.dtd">
+        <!DOCTYPE session SYSTEM "thuban-1.1.dtd">
         <session title="single map&amp;layer"
-           xmlns="http://thuban.intevation.org/dtds/thuban-1.0.0.dtd">
+           xmlns="http://thuban.intevation.org/dtds/thuban-1.1-dev.dtd">
             <fileshapesource id="D1"
                 filename="../../Data/iceland/political.shp"
                 filetype="shapefile"/>
@@ -215,14 +222,19 @@ class SaveSessionTest(unittest.TestCase, support.FileTestMixin,
                     <parameter value="proj=utm"/>
                     <parameter value="ellps=clrk66"/>
                 </projection>
-                <layer title="My Layer" shapestore="D1"
-                fill="None" stroke="#000000" stroke_width="1" visible="true">
+                <layer title="My Layer" shapestore="D1" visible="true">
                     <projection name="Layer Projection">
                         <parameter value="proj=lcc"/>
                         <parameter value="ellps=clrk66"/>
                         <parameter value="lat_1=0"/>
                         <parameter value="lat_2=20"/>
                     </projection>
+                    <classification>
+                        <clnull label="">
+                            <cldata fill="None" stroke="#000000"
+		    	        stroke_width="1"/>
+                        </clnull>
+                    </classification>
                 </layer>
             </map>
         </session>'''
@@ -234,40 +246,58 @@ class SaveSessionTest(unittest.TestCase, support.FileTestMixin,
         self.validate_data(written_contents)
 
     def testRasterLayer(self):
-        # deliberately put an apersand in the title :)
-        session = Session("single map&layer")
-        map = Map("Test Map")
-        session.AddMap(map)
-        # use shapefile from the example data
-        imgfile = os.path.join(os.path.dirname(__file__),
-                               os.pardir, "Data", "iceland", "island.tif")
-        layer = RasterLayer("My RasterLayer", imgfile)
-        map.AddLayer(layer)
 
-        filename = self.temp_file_name("%s.thuban" % self.id())
-        save_session(session, filename)
-        session.Destroy()
+        MASK_NONE = RasterLayer.MASK_NONE
+        MASK_BIT = RasterLayer.MASK_BIT
+        MASK_ALPHA = RasterLayer.MASK_ALPHA
 
-        file = open(filename)
-        written_contents = file.read()
-        file.close()
-        expected_contents = '''<?xml version="1.0" encoding="UTF-8"?>
-        <!DOCTYPE session SYSTEM "thuban-1.0.dtd">
-        <session title="single map&amp;layer"
-           xmlns="http://thuban.intevation.org/dtds/thuban-1.0.0.dtd">
-            <map title="Test Map">
-                <rasterlayer title="My RasterLayer"
-                        filename="../../Data/iceland/island.tif"
-                        visible="true">
-                </rasterlayer>
-            </map>
-        </session>'''
-        #print written_contents
-        #print "********************************************"
-        #print expected_contents
-        self.compare_xml(written_contents, expected_contents)
+        for opacity, masktype, opname, maskname in \
+            [(1,  MASK_BIT,   '', ''),
+             (.2, MASK_BIT,   'opacity="0.2"', ''),
+             (1,  MASK_ALPHA, '',              'masktype="alpha"'),
+             (.5, MASK_ALPHA, 'opacity="0.5"', 'masktype="alpha"'),
+             (1,  MASK_NONE,  '',              'masktype="none"'),
+             (0,  MASK_NONE,  'opacity="0"',   'masktype="none"') ]:
 
-        self.validate_data(written_contents)
+
+            # deliberately put an apersand in the title :)
+            session = Session("single map&layer")
+            map = Map("Test Map")
+            session.AddMap(map)
+            # use shapefile from the example data
+            imgfile = os.path.join(os.path.dirname(__file__),
+                                   os.pardir, "Data", "iceland", "island.tif")
+            layer = RasterLayer("My RasterLayer", imgfile)
+
+            layer.SetOpacity(opacity)
+            layer.SetMaskType(masktype)
+
+            map.AddLayer(layer)
+
+            filename = self.temp_file_name("%s.thuban" % self.id())
+            save_session(session, filename)
+            session.Destroy()
+
+            file = open(filename)
+            written_contents = file.read()
+            file.close()
+            expected_contents = '''<?xml version="1.0" encoding="UTF-8"?>
+            <!DOCTYPE session SYSTEM "thuban-1.1.dtd">
+            <session title="single map&amp;layer"
+               xmlns="http://thuban.intevation.org/dtds/thuban-1.1-dev.dtd">
+                <map title="Test Map">
+                    <rasterlayer title="My RasterLayer"
+                            filename="../../Data/iceland/island.tif"
+                            visible="true" %s %s>
+                    </rasterlayer>
+                </map>
+            </session>''' % (opname, maskname)
+            #print written_contents
+            #print "********************************************"
+            #print expected_contents
+            self.compare_xml(written_contents, expected_contents)
+
+            self.validate_data(written_contents)
 
     def testClassifiedLayer(self):
         """Save a session with a single map with classifications"""
@@ -307,10 +337,14 @@ class SaveSessionTest(unittest.TestCase, support.FileTestMixin,
         layer2.SetClassificationColumn("POPYCOUN")
 
         # Classification with Latin 1 text
-        clazz.AppendGroup(ClassGroupSingleton('\xe4\xf6\xfc', # ae, oe, ue
-                                              ClassGroupProperties(),
-                                              '\xdcml\xe4uts')) # Uemlaeuts
+        clazz.AppendGroup(ClassGroupSingleton(
+            internal_from_unicode(u'\xe4\xf6\xfc'), # ae, oe, ue
+            ClassGroupProperties(),
+            internal_from_unicode(u'\xdcml\xe4uts'))) # Uemlaeuts
 
+        # Pattern
+        clazz.AppendGroup(ClassGroupPattern("BUI", ClassGroupProperties(),
+                                            "pattern")) 
 
         filename = self.temp_file_name("%s.thuban" % self.id())
         save_session(session, filename)
@@ -319,9 +353,9 @@ class SaveSessionTest(unittest.TestCase, support.FileTestMixin,
         written_contents = file.read()
         file.close()
         expected_contents = '''<?xml version="1.0" encoding="UTF-8"?>
-        <!DOCTYPE session SYSTEM "thuban-1.0.dtd">
+        <!DOCTYPE session SYSTEM "thuban-1.1.dtd">
         <session title="Map with Classifications"
-           xmlns="http://thuban.intevation.org/dtds/thuban-1.0.0.dtd">
+           xmlns="http://thuban.intevation.org/dtds/thuban-1.1-dev.dtd">
             <fileshapesource id="D1"
                 filename="../../Data/iceland/political.shp"
                 filetype="shapefile"/>
@@ -331,8 +365,7 @@ class SaveSessionTest(unittest.TestCase, support.FileTestMixin,
                     <parameter value="proj=utm"/>
                     <parameter value="ellps=clrk66"/>
                 </projection>
-                <layer title="My Layer" shapestore="D1"
-                fill="None" stroke="#000000" stroke_width="1" visible="true">
+                <layer title="My Layer" shapestore="D1" visible="true">
                     <classification field="AREA" field_type="double">
                         <clnull label="">
                             <cldata fill="None" stroke="#000000" stroke_width="1"/>
@@ -351,8 +384,7 @@ class SaveSessionTest(unittest.TestCase, support.FileTestMixin,
                         </clrange>
                     </classification>
                 </layer>
-                <layer title="My Layer" shapestore="D1"
-                fill="None" stroke="#000000" stroke_width="1" visible="true">
+                <layer title="My Layer" shapestore="D1" visible="true">
                     <classification field="POPYCOUN" field_type="string">
                         <clnull label="">
                             <cldata fill="None" stroke="#000000" stroke_width="1"/>
@@ -361,6 +393,9 @@ class SaveSessionTest(unittest.TestCase, support.FileTestMixin,
                              label="\xc3\x9cml\xc3\xa4uts">
                             <cldata fill="None" stroke="#000000" stroke_width="1"/>
                         </clpoint>
+                        <clpattern pattern="BUI" label="pattern">
+                            <cldata fill="None" stroke="#000000" stroke_width="1"/>
+                        </clpattern>
                     </classification>
                 </layer>
             </map>
@@ -390,9 +425,9 @@ class SaveSessionTest(unittest.TestCase, support.FileTestMixin,
         written_contents = file.read()
         file.close()
         expected_contents = '''<?xml version="1.0" encoding="UTF-8"?>
-        <!DOCTYPE session SYSTEM "thuban-1.0.dtd">
+        <!DOCTYPE session SYSTEM "thuban-1.1.dtd">
         <session title="a DBF Table session"
-           xmlns="http://thuban.intevation.org/dtds/thuban-1.0.0.dtd">
+           xmlns="http://thuban.intevation.org/dtds/thuban-1.1-dev.dtd">
             <filetable id="D1" filename="../../Data/iceland/political.dbf"
                 filetype="DBF" title="political"/>
         </session>'''
@@ -448,9 +483,9 @@ class SaveSessionTest(unittest.TestCase, support.FileTestMixin,
             written_contents = file.read()
             file.close()
             expected_contents = '''<?xml version="1.0" encoding="UTF-8"?>
-            <!DOCTYPE session SYSTEM "thuban-1.0.dtd">
+            <!DOCTYPE session SYSTEM "thuban-1.1.dtd">
             <session title="A Joined Table session"
-             xmlns="http://thuban.intevation.org/dtds/thuban-1.0.0.dtd">
+             xmlns="http://thuban.intevation.org/dtds/thuban-1.1-dev.dtd">
                 <fileshapesource filename="../../Data/iceland/roads-line.shp"
                                  filetype="shapefile" id="D142197204"/>
                 <filetable filename="save_joinedtable.dbf"
@@ -466,8 +501,14 @@ class SaveSessionTest(unittest.TestCase, support.FileTestMixin,
                                     shapesource="D142197204"/>
                 <map title="Test Map">
                     <layer title="My Layer"
-                           shapestore="D141915644" visible="true"
-                           stroke="#000000" stroke_width="1" fill="None"/>
+                           shapestore="D141915644" visible="true">
+                        <classification>
+                            <clnull label="">
+                                <cldata fill="None" stroke="#000000"
+				        stroke_width="1"/>
+                            </clnull>
+                        </classification>
+                    </layer>
                 </map>
             </session>'''
 
@@ -491,6 +532,13 @@ class SaveSessionTest(unittest.TestCase, support.FileTestMixin,
             def _fetch_table_information(self):
                 # pretend that we've found a geometry column
                 self.geometry_column = "the_geom"
+		# pretend this is a ARC shape type.
+		self.shape_type = SHAPETYPE_ARC
+            def IDColumn(self):
+                """Return an object with a name attribute with value 'gid'"""
+                class dummycol:
+                    name = "gid"
+                return dummycol
 
         session = Session("A PostGIS Session")
         try:
@@ -513,18 +561,25 @@ class SaveSessionTest(unittest.TestCase, support.FileTestMixin,
             written = file.read()
             file.close()
             expected = '''<?xml version="1.0" encoding="UTF-8"?>
-            <!DOCTYPE session SYSTEM "thuban-1.0.dtd">
+            <!DOCTYPE session SYSTEM "thuban-1.1.dtd">
             <session title="A PostGIS Session"
-             xmlns="http://thuban.intevation.org/dtds/thuban-1.0.0.dtd">
+             xmlns="http://thuban.intevation.org/dtds/thuban-1.1-dev.dtd">
                 <dbconnection id="DB"
                               dbtype="postgis" dbname="plugh"
                               host="xyzzy" port="42"
                               user="grue"/>
-                <dbshapesource id="roads" dbconn="DB" tablename="roads"/>
+                <dbshapesource id="roads" dbconn="DB" tablename="roads"
+                               id_column="gid" geometry_column="the_geom"/>
                 <map title="Test Map">
                     <layer title="Roads to Nowhere"
-                           shapestore="roads" visible="true"
-                           stroke="#000000" stroke_width="1" fill="None"/>
+                           shapestore="roads" visible="true">
+                        <classification>
+                            <clnull label="">
+                                <cldata fill="None" stroke="#000000"
+		    	            stroke_width="1"/>
+                            </clnull>
+                        </classification>
+                    </layer>
                 </map>
             </session>'''
             self.compare_xml(written, expected)
